# -*- coding: utf-8 -*-
r"""
2d substitutions

EXAMPLES::

    sage: from slabbe import Substitution2d
    sage: A = [[0,1],[2,3]]
    sage: B = [[4,5]]
    sage: d = {0:A, 1:B}
    sage: Substitution2d(d)
    Substitution 2d: {0: [[0, 1], [2, 3]], 1: [[4, 5]]}
"""
#*****************************************************************************
#       Copyright (C) 2017 Sebastien Labbe <slabqc@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************
from __future__ import absolute_import, print_function
import itertools

from sage.misc.decorators import rename_keyword
from sage.misc.superseded import deprecated_function_alias

class Substitution2d(object):
    r"""
    INPUT:

    - ``d`` -- dict, key -> value, where each value is a table such that
      table[x][y] refers to the tile at position (x,y) in cartesian
      coordinates (*not* in the matrix-like coordinates)

    EXAMPLES::

        sage: from slabbe import Substitution2d
        sage: A = [[0,1],[2,3]]
        sage: B = [[4,5]]
        sage: d = {0:A, 1:B}
        sage: s = Substitution2d(d)
        sage: s
        Substitution 2d: {0: [[0, 1], [2, 3]], 1: [[4, 5]]}
    """
    def __init__(self, d):
        r"""
        See class documentation.

        EXAMPLES::

            sage: from slabbe import Substitution2d
            sage: A = [[0,1],[2,3]]
            sage: B = [[4,5]]
            sage: d = {0:A, 1:B}
            sage: s = Substitution2d(d)
        """
        self._d = d

    def __repr__(self):
        r"""
        EXAMPLES::

            sage: from slabbe import Substitution2d
            sage: A = [[0,1],[2,3]]
            sage: B = [[4,5]]
            sage: d = {0:A, 1:B}
            sage: s = Substitution2d(d)
            sage: s
            Substitution 2d: {0: [[0, 1], [2, 3]], 1: [[4, 5]]}
        """
        return "Substitution 2d: {}".format(self._d)

    def __eq__(self, other):
        r"""
        INPUT:

        - ``other`` -- substitution 2d

        EXAMPLES::

            sage: from slabbe import Substitution2d
            sage: A = [[0,1],[2,3]]
            sage: B = [[4,5]]
            sage: d = {0:A, 1:B}
            sage: s = Substitution2d(d)
            sage: s == Substitution2d(d)
            True
        """
        return isinstance(other, Substitution2d) and self._d == other._d

    def _latex_(self, ncolumns=8, align='l', variableA=None,
            variableB=None):
        r"""
        INPUT:

        - ``ncolumns`` -- integer
        - ``align`` -- character (default:``'l'``), latex alignment symbol
          ``'l'``, ``'r'`` or ``'c'``.
        - ``variableA`` -- string or ``None``
        - ``variableB`` -- string or ``None``

        EXAMPLES::

            sage: from slabbe import Substitution2d
            sage: A = [[0,1],[2,3]]
            sage: B = [[4,5]]
            sage: d = {0:A, 1:B}
            sage: s = Substitution2d(d)
            sage: latex(s)
            \begin{array}{llllllll}
            0\mapsto \left(\begin{array}{rr}
            1 & 3 \\
            0 & 2
            \end{array}\right)
            ,&
            1\mapsto \left(\begin{array}{r}
            5 \\
            4
            \end{array}\right)
            .
            \end{array}

        ::

            sage: s._latex_(2, 'c', variableA='x', variableB='y')
            \begin{array}{cc}
            x_{0}\mapsto \left(\begin{array}{rr}
            y_{1} & y_{3} \\
            y_{0} & y_{2}
            \end{array}\right)
            ,&
            x_{1}\mapsto \left(\begin{array}{r}
            y_{5} \\
            y_{4}
            \end{array}\right)
            .
            \end{array}

        """
        from sage.matrix.constructor import matrix
        from sage.modules.free_module_element import vector
        from sage.misc.latex import latex
        from sage.misc.latex import LatexExpr
        from sage.calculus.var import var
        from slabbe.matrices import map_coefficients_to_variable_index
        lines = []
        lines.append(r'\begin{{array}}{{{}}}'.format(align*ncolumns))
        for i,(key,table) in enumerate(self._d.items()):
            M = matrix.column([col[::-1] for col in table])
            if M.nrows() == 1:
                # do not latex array for horizontal vectors
                M = vector(M)
            if variableA:
                key = var('{}_{}'.format(variableA, key))
            if variableB:
                M = map_coefficients_to_variable_index(M, variableB)
            lines.append(r'{}\mapsto {}'.format(latex(key), latex(M)))
            if (i+1) == len(self._d):
                lines.append(r'.')
            elif (i+1) % ncolumns == 0:
                lines.append(r',\\')
            else:
                lines.append(r',&')
        lines.append(r'\end{array}')
        return LatexExpr('\n'.join(lines))

    @classmethod
    def from_1d_row_substitution(self, s):
        r"""
        INPUT:

        - ``s`` -- dict

        EXAMPLES::

            sage: from slabbe import Substitution2d
            sage: fibo = {0:[0,1], 1:[0]}
            sage: s = Substitution2d.from_1d_row_substitution(fibo)
            sage: s
            Substitution 2d: {0: [[0], [1]], 1: [[0]]}
        """
        d = {key:[[a] for a in row] for key,row in s.items()}
        return Substitution2d(d)

    @classmethod
    def from_1d_column_substitution(self, s):
        r"""
        INPUT:

        - ``s`` -- dict

        EXAMPLES::

            sage: from slabbe import Substitution2d
            sage: fibo = {0:[0,1], 1:[0]}
            sage: s = Substitution2d.from_1d_column_substitution(fibo)
            sage: s
            Substitution 2d: {0: [[0, 1]], 1: [[0]]}
        """
        d = {key:[[a for a in col]] for key,col in s.items()}
        return Substitution2d(d)

    @classmethod
    def from_permutation(self, d):
        r"""
        INPUT:

        - ``d`` -- dict

        EXAMPLES::

            sage: from slabbe import Substitution2d
            sage: s = Substitution2d.from_permutation({4:0, 5:1})
            sage: s
            Substitution 2d: {4: [[0]], 5: [[1]]}

        ::

            sage: A = [[5,6],[7,8]]
            sage: B = [[6,5],[9,8]]
            sage: t = Substitution2d({0:A, 1:B})
            sage: t
            Substitution 2d: {0: [[5, 6], [7, 8]], 1: [[6, 5], [9, 8]]}
            sage: t*s
            Substitution 2d: {4: [[5, 6], [7, 8]], 5: [[6, 5], [9, 8]]}

        ::

            sage: u = Substitution2d.from_permutation({5:0, 6:1, 7:2, 8:3, 9:4})
            sage: u
            Substitution 2d: {5: [[0]], 6: [[1]], 7: [[2]], 8: [[3]], 9: [[4]]}
            sage: u * t
            Substitution 2d: {0: [[0, 1], [2, 3]], 1: [[1, 0], [4, 3]]}
        """
        d = {key:[[d[key]]] for key in sorted(d)}
        return Substitution2d(d)

    def call_on_row(self, row):
        r"""
        INPUT:

        - ``row`` -- list

        EXAMPLES::

            sage: from slabbe import Substitution2d
            sage: A = [[0,1],[2,3]]
            sage: B = [[4,5]]
            sage: C = [[6,7,8]]
            sage: d = {0:A, 1:B, 2:C}
            sage: s = Substitution2d(d)
            sage: row = [0,1,1,0]
            sage: s.call_on_row(row)
            [[0, 1], [2, 3], [4, 5], [4, 5], [0, 1], [2, 3]]
            sage: s.call_on_row([2])
            [[6, 7, 8]]

        TESTS::

            sage: s.call_on_row([])
            []
            sage: s.call_on_row([1,2])
            Traceback (most recent call last):
            ...
            ValueError: the image of the row contains columns of different height (={2, 3})
        """
        if not row:
            return []
        columns = []
        for r in row:
            for column in self._d[r]:
                columns.append(column)
        s = set(map(len,columns))
        if len(s) != 1:
            raise ValueError("the image of the row contains columns of "
                    "different height (={})".format(s))
        return columns

    def call_on_column(self, column, heights=None):
        r"""
        INPUT:

        - ``column`` -- list
        - ``heights`` -- None or list (default: ``None``)

        EXAMPLES::

            sage: from slabbe import Substitution2d
            sage: A = [[0,1],[2,3]]
            sage: B = [[4],[5]]
            sage: C = [[6,7,8]]
            sage: d = {0:A, 1:B, 2:C}
            sage: s = Substitution2d(d)
            sage: s.call_on_column([0])
            [[0, 1], [2, 3]]
            sage: s.call_on_column([0,1])
            [[0, 1, 4], [2, 3, 5]]
            sage: s.call_on_column([0,1,1,0,0])
            [[0, 1, 4, 4, 0, 1, 0, 1], [2, 3, 5, 5, 2, 3, 2, 3]]

        It can compute the image of columns with ``None`` as entries::

            sage: s.call_on_column([0,None], heights=[2,3])
            [[0, 1, None, None, None], [2, 3, None, None, None]]
            sage: s.call_on_column([0,None], heights=[2,2])
            [[0, 1, None, None], [2, 3, None, None]]
            sage: s.call_on_column([None], heights=[3])
            [[None, None, None]]

        TESTS::

            sage: s.call_on_column([])
            []
            sage: s.call_on_column([0,2])
            Traceback (most recent call last):
            ...
            ValueError: the image of 2 in the column (=[0, 2]) has width 1
            but the image of another has width 2

        """
        if not column:
            return []

        # compute the width of the image
        for a in column:
            if not a is None:
                width = len(self._d[a])
                break
        else:
            width = 1

        # compute the image
        rep = [[] for _ in range(width)]
        for i,a in enumerate(column):
            if a is None:
                if heights is None:
                    raise ValueError("the {}-th element of given column is None,"
                            " so you must provide the list ``heights`` as"
                            " input".format(i))
                height = heights[i]
                for j in range(width):
                    rep[j].extend([None]*height)
            else:
                image_a = self._d[a]
                if not len(image_a) == width:
                    raise ValueError("the image of {} in the column (={}) has"
                            " width {} but the image of another"
                            " has width {}".format(a, column, len(image_a), width))
                if heights and not len(image_a[0]) == heights[i]:
                    raise ValueError("the image of {} in the column (={}) has"
                            " height {} but inputs ``heights`` says it should be"
                            " {}".format(a, column, len(image_a[0]), heights[i]))
                for j,col in enumerate(image_a):
                    rep[j].extend(col)
        return rep

    def __call__(self, table, order=1):
        r"""
        INPUT:

        - ``table`` -- list of list, such that table[x][y] refers to the
          tile at position (x,y) in cartesian coordinates (*not* in the
          matrix-like coordinates)
        -  ``order`` - integer or plus ``Infinity`` (default: 1)

        TODO: implement another call on table with matrix like coordinates

        OUTPUT:

            list of columns

        EXAMPLES::

            sage: from slabbe import Substitution2d
            sage: A = [[0,1],[2,3]]
            sage: B = [[4,5],[6,7]]
            sage: C = [[8,9]]
            sage: d = {0:A, 1:B, 2:C}
            sage: s = Substitution2d(d)
            sage: table = [[0,1],[1,1]]
            sage: s(table)
            [[0, 1, 4, 5], [2, 3, 6, 7], [4, 5, 4, 5], [6, 7, 6, 7]]

        ::

            sage: A = [[0,1],[2,0]]
            sage: B = [[2,1],[2,0]]
            sage: C = [[1,2],[1,1]]
            sage: d = {0:A, 1:B, 2:C}
            sage: s = Substitution2d(d)
            sage: s([[0]])
            [[0, 1], [2, 0]]
            sage: s([[0]],2)
            [[0, 1, 2, 1], [2, 0, 2, 0], [1, 2, 0, 1], [1, 1, 2, 0]]
            sage: s([[0]],3)
            [[0, 1, 2, 1, 1, 2, 2, 1],
             [2, 0, 2, 0, 1, 1, 2, 0],
             [1, 2, 0, 1, 1, 2, 0, 1],
             [1, 1, 2, 0, 1, 1, 2, 0],
             [2, 1, 1, 2, 0, 1, 2, 1],
             [2, 0, 1, 1, 2, 0, 2, 0],
             [2, 1, 2, 1, 1, 2, 0, 1],
             [2, 0, 2, 0, 1, 1, 2, 0]]

        Works if some ``None`` entries are involved::

            sage: s([[None]])
            [[None]]
            sage: s([[None],[0]])
            [[None, None], [0, 1], [2, 0]]
            sage: s([[None,0],[0,2]])
            [[None, None, 0, 1], [None, None, 2, 0], [0, 1, 1, 2], [2, 0, 1, 1]]

        TESTS::

            sage: A = [[0,1],[2,3]]
            sage: B = [[4,5],[6,7]]
            sage: C = [[8,9]]
            sage: d = {0:A, 1:B, 2:C}
            sage: s = Substitution2d(d)
            sage: s([])
            []
            sage: s([[0,1], [1,2]])
            Traceback (most recent call last):
            ...
            ValueError: the image of 2 in the column (=[1, 2]) has width 1
            but the image of another has width 2
        """
        if not table:
            return []

        if order == 1:
            # compute the heights of the image of each row
            # in case their are some None entries involved
            heights = []
            for i in range(len(table[0])):
                for col in table:
                    if col[i] is not None:
                        height = len(self._d[col[i]][0])
                        break
                else:
                    height = 1
                heights.append(height)
            # compute the image
            columns = []
            for col in table:
                col_image = self.call_on_column(col, heights)
                columns.extend(col_image)
            return columns
        elif order > 1:
            return self(self(table, order-1))
        elif order == 0:
            return table
        else:
            raise TypeError("order (%s) must be a positive integer" % order)

    def __mul__(self, other):
        r"""
        INPUT:

        - ``other`` -- substitution 2d

        EXAMPLES::

            sage: from slabbe import Substitution2d
            sage: A = [[0,1],[0,1]]
            sage: B = [[1,0],[1,1]]
            sage: d = {0:A, 1:B}
            sage: s = Substitution2d(d)
            sage: s
            Substitution 2d: {0: [[0, 1], [0, 1]], 1: [[1, 0], [1, 1]]}
            sage: s * s
            Substitution 2d: {0: [[0, 1, 1, 0], [0, 1, 1, 1], [0, 1, 1, 0], [0, 1, 1, 1]], 1: [[1, 0, 0, 1], [1, 1, 0, 1], [1, 0, 1, 0], [1, 1, 1, 1]]}
            sage: s * s * s
            Substitution 2d: {0: [[0, 1, 1, 0, 1, 0, 0, 1], [0, 1, 1, 1, 1, 1, 0, 1], [0, 1, 1, 0, 1, 0, 1, 0], [0, 1, 1, 1, 1, 1, 1, 1], [0, 1, 1, 0, 1, 0, 0, 1], [0, 1, 1, 1, 1, 1, 0, 1], [0, 1, 1, 0, 1, 0, 1, 0], [0, 1, 1, 1, 1, 1, 1, 1]], 1: [[1, 0, 0, 1, 0, 1, 1, 0], [1, 1, 0, 1, 0, 1, 1, 1], [1, 0, 1, 0, 0, 1, 1, 0], [1, 1, 1, 1, 0, 1, 1, 1], [1, 0, 0, 1, 1, 0, 0, 1], [1, 1, 0, 1, 1, 1, 0, 1], [1, 0, 1, 0, 1, 0, 1, 0], [1, 1, 1, 1, 1, 1, 1, 1]]}

        ::

            sage: t = Substitution2d.from_permutation({0:4, 1:5})
            sage: t * s
            Substitution 2d: {0: [[4, 5], [4, 5]], 1: [[5, 4], [5, 5]]}
            sage: s * t
            Traceback (most recent call last):
            ...
            ValueError: codomain alphabet of other (={4, 5}) must be
            included in domain alphabet of self (={0, 1})
        """
        if not self.domain_alphabet() >= other.codomain_alphabet():
            raise ValueError("codomain alphabet of other (={}) must be included"
                " in domain alphabet of self (={})".format(other.codomain_alphabet(),
                                                            self.domain_alphabet()))
        d = {}
        for a,image_a in other._d.items():
            d[a] = self(image_a)
        return Substitution2d(d)

    def reversal(self):
        r"""
        Return the reversal of self.

        EXAMPLES::

            sage: from slabbe import Substitution2d
            sage: A = [[1,2],[3,4]]
            sage: B = [[5,6],[7,8]]
            sage: d = {0:A, 1:B}
            sage: s = Substitution2d(d)
            sage: s.reversal()
            Substitution 2d: {0: [[4, 3], [2, 1]], 1: [[8, 7], [6, 5]]}
        """
        d = {}
        for a,image_a in self._d.items():
            d[a] = [col[::-1] for col in reversed(image_a)]
        return Substitution2d(d)

    def inverse(self):
        r"""
        Return the inverse of self (when self is a permutation).

        EXAMPLES::

            sage: from slabbe import Substitution2d
            sage: d = {0:7, 1:8}
            sage: s = Substitution2d.from_permutation(d)
            sage: s
            Substitution 2d: {0: [[7]], 1: [[8]]}
            sage: s.inverse()
            Substitution 2d: {7: [[0]], 8: [[1]]}

        TESTS::

            sage: s = Substitution2d({8: [[1]], 7: [[0,1]]})
            sage: s.inverse()
            Traceback (most recent call last):
            ...
            ValueError: self must be a permutation but image of 7 is [[0, 1]]

        """
        d = {}
        for a,image_a in self._d.items():
            if len(image_a) > 1 or len(image_a[0]) > 1:
                raise ValueError('self must be a permutation but image '
                                 'of {} is {}'.format(a, image_a))
            d[image_a[0][0]] = a 
        return Substitution2d.from_permutation(d)

    def letter_to_letter_dict(self, pos=(0,0)):
        r"""
        Return the inverse of self (when self is a permutation).

        INPUT:

        - ``pos`` -- tuple (default:``(0,0)``), tuple of two integers

        EXAMPLES::

            sage: from slabbe import Substitution2d
            sage: A = [[0,1],[2,3]]
            sage: B = [[4,5]]
            sage: s = Substitution2d({0:A, 1:B})
            sage: s
            Substitution 2d: {0: [[0, 1], [2, 3]], 1: [[4, 5]]}
            sage: s.letter_to_letter_dict(pos=(0,0))
            {0: 0, 1: 4}

        """
        x,y = pos
        return {a:image_a[x][y] for a,image_a in self._d.items()}

    def apply_matrix_transformation(self, M):
        r"""
        INPUT:

        - ``M`` -- matrix in SL(2,Z)

        EXAMPLES::

            sage: from slabbe import Substitution2d
            sage: A = [[0,1],[0,1]]
            sage: B = [[1,0],[1,1]]
            sage: d = {0:A, 1:B}
            sage: s = Substitution2d(d)
            sage: M = matrix(2, (1,1,0,1))
            sage: s
            Substitution 2d: {0: [[0, 1], [0, 1]], 1: [[1, 0], [1, 1]]}
            sage: s.apply_matrix_transformation(M)
            Substitution 2d: {0: [[0, None], [0, 1], [None, 1]], 1: [[1, None], [1, 0], [None, 1]]}

        """
        from slabbe.wang_tiles import WangTiling
        d = {}
        for a,image_a in self._d.items():
            d[a] = WangTiling(image_a,tiles=[]).apply_matrix_transformation(M).table()
        return Substitution2d(d)

    def domain_alphabet(self):
        r"""
        EXAMPLES::

            sage: from slabbe import Substitution2d
            sage: A = [[5,6],[7,8]]
            sage: B = [[6,5],[9,8]]
            sage: d = {0:A, 1:B}
            sage: s = Substitution2d(d)
            sage: s.domain_alphabet()
            {0, 1}
        """
        return set(self._d.keys())

    def codomain_alphabet(self):
        r"""
        EXAMPLES::

            sage: from slabbe import Substitution2d
            sage: A = [[5,6],[7,8]]
            sage: B = [[6,5],[9,8]]
            sage: d = {0:A, 1:B}
            sage: s = Substitution2d(d)
            sage: s.codomain_alphabet()
            {5, 6, 7, 8, 9}

        Blank ``None`` are ignored::

            sage: A = [[5,6],[7,8]]
            sage: B = [[6,5],[9,None]]
            sage: d = {0:A, 1:B}
            sage: s = Substitution2d(d)
            sage: s.codomain_alphabet()
            {5, 6, 7, 8, 9}
        """
        s = set()
        for a,image_a in self._d.items():
            for column in image_a:
                s.update(column)
        if None in s:
            s.remove(None)
        return s

    def incidence_matrix(self):
        r"""
        Return the incidence matrix of self.

        Some default ordering (sorted) is used for the domain and codomain
        alphabet.

        EXAMPLES::

            sage: from slabbe import Substitution2d
            sage: A = [[0,1],[2,3]]
            sage: B = [[4,5]]
            sage: C = [[6,7,8]]
            sage: d = {0:A, 1:B, 2:C}
            sage: s = Substitution2d(d)
            sage: s.incidence_matrix()
            [1 0 0]
            [1 0 0]
            [1 0 0]
            [1 0 0]
            [0 1 0]
            [0 1 0]
            [0 0 1]
            [0 0 1]
            [0 0 1]
        """
        from collections import Counter
        from sage.matrix.constructor import matrix
        domain_alphabet = sorted(self.domain_alphabet())
        codomain_alphabet = sorted(self.codomain_alphabet())
        L = []
        for a in domain_alphabet:
            c = Counter()
            image_a = self._d[a]
            for column in image_a:
                c.update(column)
            v = [c[b] for b in codomain_alphabet]
            L.append(v)
        return matrix.column(L)

    def desubstitute(self, tiles, function=None):
        r"""
        Return the Wang tile set obtained from the desubstitution of the
        given Wang tile set.

        INPUT:

        - ``tiles`` -- list of Wang tiles, each tile being a 4-tuple of
          (east, north, west, south) colors
        - ``fn`` -- a function (default: ``None``) to apply to the
          new colors which are tuple of previous colors

        OUTPUT:

            dict, key -> tile

        """
        raise NotImplementedError('method desubstitute moved to wang tile')

    @rename_keyword(fontsize='font')
    def wang_tikz(self, domain_tiles, codomain_tiles, domain_color=None,
            codomain_color=None, size=1, scale=1, font=r'\normalsize',
            rotate=None, label_shift=.2, id=True, edges=True, 
            ncolumns=4, direction='right', extra_space=1):
        r"""
        Return the tikz code showing what the substitution A->B* does on
        Wang tiles.

        INPUT:

        - ``domain_tiles`` -- tiles of the domain
        - ``codomain_tiles`` -- tiles of the codomain
        - ``domain_color`` -- dict (default: ``None``) from tile values -> tikz colors
        - ``codomain_color`` -- dict (default: ``None``) from tile values -> tikz colors
        - ``size`` -- number (default: ``1``), size of the tile
        - ``scale`` -- number (default: ``1``), scale of tikzpicture
        - ``font`` -- string (default: ``r'\normalsize'``
        - ``rotate`` -- list or ``None`` (default:``None``) list of four angles
          in degrees like ``(0,0,0,0)``, the rotation angle to apply to each
          label of Wang tiles. If ``None``, it performs a 90 degres rotation
          for left and right labels taking more than one character.
        - ``label_shift`` -- number (default: ``.2``) translation distance
          of the label from the edge
        - ``id`` -- boolean (default: ``True``), presence of the tile id
        - ``ncolumns`` -- integer (default: ``4``)
        - ``edges`` -- bool (default: ``True``) 
        - ``direction`` -- string (default: ``'right'``) or ``'down'``
        - ``extra_space`` -- number (default: ``1``), space between the
          tile and its image

        OUTPUT:

            dict, key -> tile

        EXAMPLES::

            sage: from slabbe import WangTileSet, Substitution2d
            sage: A = [[0,1,2],[1,0,0]]
            sage: B = [[0,1,2]]
            sage: d = {4:A, 5:B}
            sage: s = Substitution2d(d)
            sage: codomain_tiles = [(0,3,1,4), (1,4,0,3), (5,6,7,8)]
            sage: W = WangTileSet(codomain_tiles)
            sage: fn = lambda colors:''.join(map(str, colors))
            sage: domain_tiles = W.desubstitute(s, fn)
            sage: tikz = s.wang_tikz(domain_tiles, codomain_tiles, rotate=(90,0,90,0))
            sage: _ = tikz.pdf(view=False)      # long time

        Applying a transformation matrix::

            sage: M = matrix(2, [1,1,0,1])
            sage: sM = s.apply_matrix_transformation(M)
            sage: tikz = sM.wang_tikz(domain_tiles, codomain_tiles)
            sage: _ = tikz.pdf(view=False)      # long time

        Down direction::

            sage: tikz = s.wang_tikz(domain_tiles, codomain_tiles,
            ....:                      direction='down')
            sage: _ = tikz.pdf(view=False)      # long time
        """
        from slabbe.wang_tiles import tile_to_tikz, WangTileSet, WangTiling
        if isinstance(codomain_tiles, WangTileSet):
            codomain_tiles = codomain_tiles._tiles
        if isinstance(domain_tiles, WangTileSet):
            domain_tiles = domain_tiles._tiles

        for a in self.codomain_alphabet():
            try:
                codomain_tiles[a]
            except IndexError:
                raise ValueError("codomain_alphabet={}, but tiles are"
                        " {}".format(self.codomain_alphabet(),
                        codomain_tiles))
        for a in self.domain_alphabet():
            try:
                domain_tiles[a]
            except IndexError:
                raise ValueError("domain_alphabet={}, but tiles are"
                        " {}".format(self.domain_alphabet(),
                        domain_tiles))

        if not direction in ['right', 'down']:
            raise ValueError("direction(={}) must be 'right' or"
                    " 'down'".format(direction))

        lines = []
        lines.append(r'\begin{tikzpicture}')
        lines.append(r'[scale={}]'.format(scale))
        lines.append(r'\tikzstyle{{every node}}=[font={}]'.format(font))
        lines.append(r'\matrix{')

        for i,a in enumerate(self._d):

            lines.append(r'\node{')
            lines.append(r'\begin{tikzpicture}')
            lines.append(r'[scale={}]'.format(scale))
            lines.append(r'\tikzstyle{{every node}}=[font={}]'.format(font))

            desubstituted_tile = domain_tiles[a] 
            this_id = a if id else None
            new_lines = tile_to_tikz(desubstituted_tile, (0,0),
                    color=domain_color, id=this_id, sizex=size, sizey=size,
                    rotate=rotate, label_shift=label_shift,
                    right_edges=edges, top_edges=edges, left_edges=edges,
                    bottom_edges=edges)
            new_lines.insert(0, r'\begin{tikzpicture}')
            new_lines.append(r'\end{tikzpicture}')
            new_lines = '\n'.join(new_lines)
            lines.append(r'\node (A) at (0,0) {{{}}};'.format(new_lines))

            image_a = self._d[a]
            tiling = WangTiling(image_a, codomain_tiles, codomain_color)
            tiling_tikz = tiling.tikz(color=codomain_color, font=font,
                    rotate=rotate, label_shift=label_shift, scale=scale,
                    edges=edges, id=id, size=size)

            size_image_x = len(image_a)
            size_image_y = len(image_a[0])

            if direction == 'right':
                xshift = extra_space + .5 * size_image_x
                yshift = 0
            elif direction == 'down':
                xshift = 0
                yshift = -extra_space - .5 * size_image_y
            lines.append(r'\node (B) at ({},{}) {{{}}};'.format(xshift, yshift,
                                             tiling_tikz.tikz_picture_code()))

            lines.append(r'\draw[-to,very thick] (A) edge (B);')

            lines.append(r'\end{tikzpicture}')
            lines.append(r'};')

            if (i+1) == len(self._d):
                lines.append(r'\\')
            elif (i+1) % ncolumns == 0:
                lines.append(r'\\')
            else:
                lines.append(r'&')

        lines.append(r'};')  # end of \matrix{
        lines.append(r'\end{tikzpicture}')

        from slabbe import TikzPicture
        return TikzPicture('\n'.join(lines), usetikzlibrary=['positioning'])

    def wang_tiles_codomain_tikz(self, codomain_tiles, color=None,
            size=1, scale=1, font=r'\normalsize', rotate=None,
            id=True, label=True, label_shift=.2, edges=True,
            ncolumns=4, direction='right'):
        r"""
        Return the tikz code of the image of the letters as a table of
        tikz tilings.

        INPUT:

        - ``domain_tiles`` -- tiles of the domain
        - ``codomain_tiles`` -- tiles of the codomain
        - ``domain_color`` -- dict (default: ``None``) from tile values -> tikz colors
        - ``codomain_color`` -- dict (default: ``None``) from tile values -> tikz colors
        - ``size`` -- number (default: ``1``), size of the tile
        - ``scale`` -- number (default: ``1``), scale of tikzpicture
        - ``font`` -- string (default: ``r'\normalsize'``
        - ``rotate`` -- list or ``None`` (default:``None``) list of four angles
          in degrees like ``(0,0,0,0)``, the rotation angle to apply to each
          label of Wang tiles. If ``None``, it performs a 90 degres rotation
          for left and right labels taking more than one character.
        - ``id`` -- boolean (default: ``True``), presence of the tile id
        - ``label`` -- boolean (default: ``True``) 
        - ``label_shift`` -- number (default: ``.2``) translation distance
          of the label from the edge
        - ``edges`` -- bool (default: ``True``) 
        - ``ncolumns`` -- integer (default: ``4``)

        OUTPUT:

            tikzpicture

        EXAMPLES::

            sage: from slabbe import WangTileSet, Substitution2d
            sage: A = [[0,1,2],[1,0,0]]
            sage: B = [[0,1,2]]
            sage: d = {4:A, 5:B}
            sage: s = Substitution2d(d)
            sage: codomain_tiles = [(0,3,1,4), (1,4,0,3), (5,6,7,8)]
            sage: W = WangTileSet(codomain_tiles)
            sage: t = s.wang_tiles_codomain_tikz(W)
            sage: _ = t.pdf(view=False)

        """
        from slabbe.wang_tiles import tile_to_tikz, WangTileSet, WangTiling
        for a in self.codomain_alphabet():
            try:
                codomain_tiles[a]
            except IndexError:
                raise ValueError("codomain_alphabet={}, but tiles are"
                        " {}".format(self.codomain_alphabet(),
                        codomain_tiles))

        lines = []
        lines.append(r'\begin{tikzpicture}')
        lines.append(r'[scale={}]'.format(scale))
        lines.append(r'\tikzstyle{{every node}}=[font={}]'.format(font))
        lines.append(r'\matrix{')
        for i,a in enumerate(self._d):

            image_a = self._d[a]
            tiling = WangTiling(image_a, codomain_tiles, color)
            tikz = tiling.tikz(color=color, font=font, id=id, label=label,
                    rotate=rotate, label_shift=label_shift, scale=scale,
                    edges=edges, size=size)

            lines.append(r'\node{{{}}};'.format(tikz.tikz_picture_code()))

            if (i+1) == len(self._d):
                lines.append(r'\\')
            elif (i+1) % ncolumns == 0:
                lines.append(r'\\')
            else:
                lines.append(r'&')

        lines.append(r'};')  # end of \matrix{
        lines.append(r'\end{tikzpicture}')

        from slabbe import TikzPicture
        return TikzPicture('\n'.join(lines))

    def list_2x2_factors(self, F=None):
        r"""
        Return the list of 2x2 factors in the associated substitutive
        shift. If a list of factors ``F`` is given, it restrict to the
        factors inside the image of ``F``.

        INPUT:

        - ``self`` -- expansive and primitive 2d substitution
        - ``F`` -- list of factors in the domain or ``None``, if given the
          output is restricted to the factors in ``F``

        OUTPUT:

            list of tables

        EXAMPLES::

            sage: from slabbe import Substitution2d
            sage: A = [[0,1],[0,1]]
            sage: B = [[1,0],[1,1]]
            sage: d = {0:A, 1:B}
            sage: s = Substitution2d(d)
            sage: sorted(s.list_2x2_factors())
            [[[0, 0], [1, 0]],
             [[0, 1], [0, 1]],
             [[0, 1], [1, 0]],
             [[0, 1], [1, 1]],
             [[1, 0], [0, 0]],
             [[1, 0], [0, 1]],
             [[1, 0], [1, 0]],
             [[1, 0], [1, 1]],
             [[1, 1], [0, 0]],
             [[1, 1], [0, 1]],
             [[1, 1], [1, 0]],
             [[1, 1], [1, 1]]]

        Restricting to the images of some factors::

            sage: sorted(s.list_2x2_factors([A]))
            [[[0, 1], [0, 1]], [[1, 0], [1, 1]], [[1, 1], [1, 0]], [[1, 1], [1, 1]]]
            sage: sorted(s.list_2x2_factors([B]))
            [[[0, 0], [1, 0]],
             [[0, 1], [0, 1]],
             [[0, 1], [1, 0]],
             [[0, 1], [1, 1]],
             [[1, 0], [0, 1]],
             [[1, 0], [1, 1]],
             [[1, 1], [1, 0]]]
            sage: sorted(s.list_2x2_factors([A,B]))
            [[[0, 0], [1, 0]],
             [[0, 1], [0, 1]],
             [[0, 1], [1, 0]],
             [[0, 1], [1, 1]],
             [[1, 0], [0, 1]],
             [[1, 0], [1, 1]],
             [[1, 1], [1, 0]],
             [[1, 1], [1, 1]]]
            sage: s.list_2x2_factors([])
            []

        """
        if not self.codomain_alphabet() <= self.domain_alphabet():
            raise ValueError("codomain alphabet (={}) is not a subset of the"
                    " domain alphabet (={})".format(self.codomain_alphabet(),
                                              self.domain_alphabet()))
        if F is None:
            from sage.sets.recursively_enumerated_set import RecursivelyEnumeratedSet
            alphabet = self.domain_alphabet()
            a = next(iter(alphabet))
            table = [[a]]
            while len(table) < 2 or len(table[0]) < 2:
                table = self(table)
            shape = [(0,0), (0,1), (1,0), (1,1)]
            seeds = set_of_factors(table, shape)
            def children(factor):
                table = [[factor[0], factor[1]],
                         [factor[2], factor[3]]]
                image = self(table)
                S = set_of_factors(image, shape)
                return S
            R = RecursivelyEnumeratedSet(seeds, children)
            return [ [[a, b], [c, d]] for (a,b,c,d) in R]
        else:
            shape = [(0,0), (0,1), (1,0), (1,1)]
            S = set()
            for table in F:
                S.update(set_of_factors(self(table), shape))
            return [ [[a, b], [c, d]] for (a,b,c,d) in S]

    def lines_alphabet(self, direction='horizontal'):
        r"""
        Return the possible alphabets on lines, i.e., the possible alphabet
        of letters that we see on a given line.

        EXAMPLES::

            sage: from slabbe import Substitution2d
            sage: A = [[0,1],[0,1]]
            sage: B = [[1,0],[1,1]]
            sage: d = {0:A, 1:B}
            sage: s = Substitution2d(d)
            sage: sorted(s.lines_alphabet())
            [(0,), (0, 1), (1,)]
            sage: sorted(s.lines_alphabet(direction='vertical'))
            [(0, 1), (1,)]

        """
        if direction == 'horizontal':
            def children(node):
                T = self.call_on_row(node)
                return [tuple(sorted(set(row))) for row in zip(*T)]
        elif direction == 'vertical':
            def children(node):
                T = self.call_on_column(node)
                return [tuple(sorted(set(column))) for column in T]
        else:
            raise ValueError("direction (={}) must be 'vertical' or"
                    " 'horizontal'".format(direction))

        from sage.sets.recursively_enumerated_set import RecursivelyEnumeratedSet
        seeds = [(a,) for a in self.domain_alphabet()]
        R = RecursivelyEnumeratedSet(seeds, children)
        G = R.to_digraph(multiedges=False)
        result = []
        for s in G.strongly_connected_components_subgraphs():
            if s.num_edges() > 0:
                for v in s.vertices():
                    result.append(v)
        return result

    def prolongable_seeds_graph(self):
        r"""
        Return the directed graph of 2x2 factors where (u,v) is an edge if
        v is the seed at the origin of the image of u under self.

        OUTPUT:

            list of tuple (2x2 matrix, integer)

        EXAMPLES::

            sage: d = {0: [[17]],
            ....:  1: [[16]],
            ....:  2: [[15], [11]],
            ....:  3: [[13], [9]],
            ....:  4: [[17], [8]],
            ....:  5: [[16], [8]],
            ....:  6: [[15], [8]],
            ....:  7: [[14], [8]],
            ....:  8: [[14, 6]],
            ....:  9: [[17, 3]],
            ....:  10: [[16, 3]],
            ....:  11: [[14, 2]],
            ....:  12: [[15, 7], [11, 1]],
            ....:  13: [[14, 6], [11, 1]],
            ....:  14: [[13, 7], [9, 1]],
            ....:  15: [[12, 6], [9, 1]],
            ....:  16: [[18, 5], [10, 1]],
            ....:  17: [[13, 4], [9, 1]],
            ....:  18: [[14, 2], [8, 0]]}
            sage: from slabbe import Substitution2d
            sage: omega = Substitution2d(d)
            sage: G = omega.prolongable_seeds_graph()
            sage: G
            Looped digraph on 50 vertices

        """
        from sage.matrix.constructor import matrix
        from sage.sets.recursively_enumerated_set import RecursivelyEnumeratedSet
        seeds = self.list_2x2_factors(F=None)
        seeds = [matrix.column(c[::-1] for c in columns) for columns in seeds]
        for m in seeds: m.set_immutable()
        def children(m):
            b,d,a,c = m.list()
            A = self([[a]])[-1][-1] # bottom left
            B = self([[b]])[-1][0] # top left
            C = self([[c]])[0][-1] # bottom right
            D = self([[d]])[0][0] # top right
            M = matrix.column(((B,A),(D,C)))
            M.set_immutable()
            return [M]
        R = RecursivelyEnumeratedSet(seeds, children)
        return R.to_digraph(multiedges=False)

    prolongable_origins = deprecated_function_alias(123456, prolongable_seeds_graph)
    def prolongable_seeds_list(self):
        r"""
        Return the list of seed which are prolongable for some power of
        self.

        OUTPUT:

            list of cycles

        EXAMPLES::

            sage: d = {0: [[17]],
            ....:  1: [[16]],
            ....:  2: [[15], [11]],
            ....:  3: [[13], [9]],
            ....:  4: [[17], [8]],
            ....:  5: [[16], [8]],
            ....:  6: [[15], [8]],
            ....:  7: [[14], [8]],
            ....:  8: [[14, 6]],
            ....:  9: [[17, 3]],
            ....:  10: [[16, 3]],
            ....:  11: [[14, 2]],
            ....:  12: [[15, 7], [11, 1]],
            ....:  13: [[14, 6], [11, 1]],
            ....:  14: [[13, 7], [9, 1]],
            ....:  15: [[12, 6], [9, 1]],
            ....:  16: [[18, 5], [10, 1]],
            ....:  17: [[13, 4], [9, 1]],
            ....:  18: [[14, 2], [8, 0]]}
            sage: from slabbe import Substitution2d
            sage: omega = Substitution2d(d)
            sage: omega.prolongable_seeds_list()
            [[
             [ 9 14]  [17 13]
             [ 1  6], [16 15]
             ],
             [
             [ 9 14]  [17 13]
             [ 8 16], [ 6  5]
             ],
             [
             [10 12]  [16 15]
             [ 9 14], [ 3  7]
             ],
             [
             [10 14]  [16 13]
             [11 17], [ 2  4]
             ]]

        """
        G = self.prolongable_seeds_graph()
        return [cycle[:-1] for cycle in G.all_simple_cycles()]

    _matrix_ = incidence_matrix
    def relabel_domain(self, other):
        r"""
        Return a permutation p such that self*p == other, if it exists.

        INPUT:

        - ``other`` -- substitution 2d

        EXAMPLES::

            sage: from slabbe import Substitution2d
            sage: A = [[0,1],[0,1]]
            sage: B = [[1,0],[1,1]]
            sage: s = Substitution2d({0:A, 1:B})
            sage: t = Substitution2d({7:A, 8:B})
            sage: s.relabel_domain(t)
            Substitution 2d: {7: [[0]], 8: [[1]]}

        TESTS::

            sage: s = Substitution2d({0:A, 1:B})
            sage: s.relabel_domain(s)
            Substitution 2d: {0: [[0]], 1: [[1]]}

        ::

            sage: s = Substitution2d({0:A, 1:B})
            sage: t = Substitution2d({7:A, 8:B, 9:[[4]]})
            sage: t.relabel_domain(s)
            Traceback (most recent call last):
            ...
            ValueError: image of letter 9 is [[4]] and is not in other
            sage: s.relabel_domain(t)
            Traceback (most recent call last):
            ...
            AssertionError: problem: self * p == other not satisfied
        """
        table_to_tuple = lambda table:tuple(tuple(col) for col in table)
        other_inv = {table_to_tuple(val):key for key,val in other._d.items()}
        p = {}
        for key,val in self._d.items():
            val_ = table_to_tuple(val)
            if val_ in other_inv:
                p[other_inv[val_]] = key 
            else:
                raise ValueError('image of letter {} is {} and is not in other'.format(key, val))
        p = Substitution2d.from_permutation(p)
        assert self * p == other, "problem: self * p == other not satisfied"
        return p

    def fixed_point_tikz(self, seed, niterations=3):
        r"""
        Return a tikz representation of a fixed point defined by the give seed.
        In the image, rectangular boxes indicate the i-th image of each seed.

        INPUT:

        - ``seed`` -- 2x2 matrix
        - ``niterations`` -- (default:``3``), number of iterations

        OUTPUT

            tikz picture

        EXAMPLES::

            sage: from slabbe import Substitution2d
            sage: A = [[1,1],[1,1]]
            sage: B = [[0,0]]
            sage: d = {0:A, 1:B}
            sage: s = Substitution2d(d)
            sage: tikz = s.fixed_point_tikz([[0,0],[0,0]])
            sage: _ = tikz.pdf()                              # not tested

        The substitution ``s`` is not prolongable, so the boxes in the
        image obtained from the square of ``s`` might better::

            sage: s2 = s*s
            sage: tikz = s2.fixed_point_tikz([[0,0],[0,0]])
            sage: _ = tikz.pdf()                              # not tested

        """
        if isinstance(seed, list):
            [[C,B],[D,A]] = seed
        else:
            # we assume it is a 2x2 matrix
            B,A,C,D = seed.list()
        quadrants = {}
        quadrants[1] = [[[A]]]
        quadrants[2] = [[[B]]]
        quadrants[3] = [[[C]]]
        quadrants[4] = [[[D]]]
        for i in [1,2,3,4]:
            for order in range(niterations):
                previous = quadrants[i][-1]
                quadrants[i].append(self(previous))
        width_heigth = lambda quad : (len(quad),len(quad[0]))
        dimq1x,dimq1y = width_heigth(quadrants[1][-1])
        dimq2x,dimq2y = width_heigth(quadrants[2][-1])
        dimq3x,dimq3y = width_heigth(quadrants[3][-1])
        dimq4x,dimq4y = width_heigth(quadrants[4][-1])
        lines = []
        lines.append(r"\begin{tikzpicture}")
        for i,col in enumerate(quadrants[1][-1]):
            for j,a in enumerate(col):
                coords = (i+.5,j+.5)
                lines.append(r"\node at {} {{{}}};".format(coords,a))
        for i,col in enumerate(quadrants[2][-1]):
            for j,a in enumerate(col):
                coords = (i-dimq2x+.5,j+.5)
                lines.append(r"\node at {} {{{}}};".format(coords,a))
        for i,col in enumerate(quadrants[3][-1]):
            for j,a in enumerate(col):
                coords = (i-dimq3x+.5,j-dimq3y+.5)
                lines.append(r"\node at {} {{{}}};".format(coords,a))
        for i,col in enumerate(quadrants[4][-1]):
            for j,a in enumerate(col):
                coords = (i+.5,j-dimq4y+.5)
                lines.append(r"\node at {} {{{}}};".format(coords,a))
        # axes
        lines.append(r"\draw {} -- {};".format((0,-dimq4y), (0,dimq1y)))
        lines.append(r"\draw {} -- {};".format((-dimq2x,0), (dimq1x,0)))
        # boxes
        for order in range(niterations):
            w,h = width_heigth(quadrants[3][order])
            lower_left = (-w, -h)
            upper_right = width_heigth(quadrants[1][order])
            lines.append(r"\draw {} rectangle {};".format(lower_left, upper_right))
        lines.append(r"\end{tikzpicture}")

        from slabbe import TikzPicture
        return TikzPicture('\n'.join(lines))

def set_of_factors(table, shape, avoid_border=0):
    r"""
    Return the set of factors of given shape in the table.

    INPUT

    - ``table`` -- list of lists
    - ``shape`` -- list, list of coordinates
    - ``avoid_border`` -- integer (default: 0), the size of the border
        to avoid during the computation

    OUTPUT:

        set of tuple of integers

    EXAMPLES::

        sage: from slabbe.substitution_2d import set_of_factors
        sage: table = [[0,1,2], [3,4,5], [6,7,8]]
        sage: set_of_factors(table, shape=[(0,0), (1,0), (0,1), (1,1)])
        {(0, 3, 1, 4), (1, 4, 2, 5), (3, 6, 4, 7), (4, 7, 5, 8)}
    """
    xmin = min(x for (x,y) in shape)
    xmax = max(x for (x,y) in shape)
    ymin = min(y for (x,y) in shape)
    ymax = max(y for (x,y) in shape)
    width = len(table)
    height = len(table[0])
    S = set()
    for i in range(0-xmin+avoid_border, width-xmax-avoid_border):
        for j in range(0-ymin+avoid_border, height-ymax-avoid_border):
            pattern = tuple(table[i+x][j+y] for (x,y) in shape)
            S.add(pattern)
    return S




# -*- coding: utf-8 -*-
r"""
Regular languages

EXAMPLES:

Language over all finite words on an alphabet::

    sage: from slabbe.language import Language
    sage: Language(alphabet=['a', 'b'])
    Language of finite words over alphabet ['a', 'b']

Finite language::

    sage: from slabbe.language import FiniteLanguage
    sage: S = ['a', 'ab', 'aab', 'aaab']
    sage: FiniteLanguage(alphabet=['a', 'b'], words=S)
    Finite language of cardinality 4 over alphabet ['a', 'b']

Regular language::

    sage: from slabbe.language import RegularLanguage
    sage: alphabet = ['a', 'b']
    sage: trans = [(0, 1, 'a'), (1, 2, 'b'), (2, 3, 'b'), (3, 4, 'a')]
    sage: automaton = Automaton(trans, initial_states=[0], final_states=[4])
    sage: RegularLanguage(alphabet, automaton)
    Regular language over ['a', 'b']
    defined by: Automaton with 5 states

Predefined languages::

    sage: from slabbe.language import languages
    sage: languages.ARP()
    Regular language over [1, 2, 3, 123, 132, 213, 231, 312, 321]
    defined by: Automaton with 7 states

AUTHORS:

 - Sébastien Labbé, initial clean and full doctested version, October 2015
"""
#*****************************************************************************
#       Copyright (C) 2015 Sébastien Labbé <slabqc@gmail.com>
#
#  Distributed under the terms of the GNU General Public License version 2 (GPLv2)
#
#  The full text of the GPLv2 is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************
from __future__ import absolute_import, print_function
import itertools
from sage.combinat.words.words import Words
from sage.combinat.finite_state_machine import Automaton

class Language(object):
    r"""
    Language of finite words

    INPUT:

    - ``alphabet`` -- iterable of letters

    EXAMPLES::

        sage: from slabbe.language import Language
        sage: Language(alphabet=['a', 'b'])
        Language of finite words over alphabet ['a', 'b']
    """
    def __init__(self, alphabet):
        r"""
        EXAMPLES::

            sage: from slabbe.language import Language
            sage: Language(alphabet=['a', 'b'])
            Language of finite words over alphabet ['a', 'b']
        """
        self._alphabet = sorted(alphabet)

    def __repr__(self):
        r"""
        EXAMPLES::

            sage: from slabbe.language import Language
            sage: Language(alphabet=['a'])
            Language of finite words over alphabet ['a']
        """
        s = "Language of finite words over alphabet {}"
        return s.format(self._alphabet)

    def words_of_length_iterator(self, length):
        r"""
        Return an iterator over words of given length.

        INPUT:

        - ``length`` -- integer

        EXAMPLES::

            sage: from slabbe.language import Language
            sage: F = Language(alphabet=['a', 'b'])
            sage: it = F.words_of_length_iterator(2)
            sage: list(it)
            [word: aa, word: ab, word: ba, word: bb]
        """
        W = Words(self._alphabet)
        return W.iterate_by_length(length)

    def complexity(self, length):
        r"""
        Returns the number of words of given length.

        .. NOTE::

            This method is defined from :func:`~words_of_length_iterator`.

        INPUT:

        - ``length`` -- integer

        EXAMPLES::

            sage: from slabbe.language import Language
            sage: F = Language(alphabet=['a', 'b'])
            sage: [F.complexity(n) for n in range(5)]
            [1, 2, 4, 8, 16]
        """
        return sum(1 for _ in self.words_of_length_iterator(length))
class FiniteLanguage(Language):
    r"""
    Finite language

    INPUT:

    - ``alphabet`` -- iterable of letters
    - ``words`` -- finite iterable of words

    EXAMPLES::

        sage: from slabbe.language import FiniteLanguage
        sage: L = ['a', 'aa', 'aaa']
        sage: FiniteLanguage(alphabet=['a'], words=L)
        Finite language of cardinality 3 over alphabet ['a']
    """
    def __init__(self, alphabet, words):
        r"""
        EXAMPLES::

            sage: from slabbe.language import FiniteLanguage
            sage: L = ['a', 'aa', 'aaa']
            sage: F = FiniteLanguage(alphabet=['a'], words=L)
        """
        Language.__init__(self, alphabet)
        self._words = words
    def __repr__(self):
        r"""
        EXAMPLES::

            sage: from slabbe.language import FiniteLanguage
            sage: L = ['a', 'ab', 'aab', 'aaab']
            sage: FiniteLanguage(alphabet=['a', 'b'], words=L)
            Finite language of cardinality 4 over alphabet ['a', 'b']
        """
        s = "Finite language of cardinality {} over alphabet {}"
        return s.format(len(self._words), self._alphabet)

    def automaton(self):
        r"""
        Return the automaton recognizing this finite language.

        EXAMPLES::

            sage: from slabbe.language import FiniteLanguage
            sage: L = ['a', 'aa', 'aaa']
            sage: F = FiniteLanguage(alphabet=['a'], words=L)
            sage: F.automaton()
            Automaton with 7 states
        """
        transitions = []
        final_states = []
        end = 1
        for w in self._words:
            start = 0
            for a in w:
                transitions.append((start, end, a))
                start, end = end, end+1
            final_states.append(start)
        return Automaton(transitions, initial_states=[0], final_states=final_states)

    def minimal_automaton(self):
        r"""
        Return the minimal automaton recognizing this finite language.

        .. NOTE:: 
        
            One of the state is not final. You may want to remove it...

        EXAMPLES::

            sage: from slabbe.language import FiniteLanguage
            sage: L = ['a', 'aa', 'aaa']
            sage: F = FiniteLanguage(alphabet=['a'], words=L)
            sage: F.minimal_automaton()
            Automaton with 5 states
        """
        return self.automaton().minimization().relabeled()

    def number_of_states(self):
        r"""
        EXAMPLES::

            sage: from slabbe.language import FiniteLanguage
            sage: L = ['a', 'aa', 'aaa']
            sage: F = FiniteLanguage(alphabet=['a'], words=L)
            sage: F.number_of_states()
            5
        """
        return len(self.minimal_automaton().states())

class RegularLanguage(Language):
    r"""
    Regular language

    INPUT:

    - ``alphabet`` -- iterable of letters
    - ``automaton`` -- finite state automaton

    EXAMPLES::

        sage: from slabbe.language import RegularLanguage
        sage: alphabet = ['a', 'b']
        sage: trans = [(0, 1, 'a'), (1, 2, 'b'), (2, 3, 'b'), (3, 4, 'a')]
        sage: automaton = Automaton(trans, initial_states=[0], final_states=[4])
        sage: RegularLanguage(alphabet, automaton)
        Regular language over ['a', 'b']
        defined by: Automaton with 5 states
    """
    def __init__(self, alphabet, automaton):
        r"""
        EXAMPLES::

            sage: from slabbe.language import RegularLanguage
            sage: alphabet = ['a', 'b']
            sage: trans = [(0, 1, 'a'), (1, 2, 'b'), (2, 3, 'b'), (3, 4, 'a')]
            sage: automaton = Automaton(trans, initial_states=[0], final_states=[4])
            sage: R = RegularLanguage(alphabet, automaton)
        """
        Language.__init__(self, alphabet)
        self._automaton = automaton


    def __repr__(self):
        r"""
        EXAMPLES::

            sage: from slabbe.language import RegularLanguage
            sage: alphabet = ['a', 'b']
            sage: trans = [(0, 1, 'a'), (1, 2, 'b'), (2, 3, 'b'), (3, 4, 'a')]
            sage: automaton = Automaton(trans, initial_states=[0], final_states=[4])
            sage: RegularLanguage(alphabet, automaton)
            Regular language over ['a', 'b']
            defined by: Automaton with 5 states
        """
        s = "Regular language over {}\ndefined by: {}"
        return s.format(self._alphabet, self._automaton)

    def words_of_length_iterator(self, length):
        r"""
        Return an iterator over words of given length.

        INPUT:

        - ``length`` -- integer

        EXAMPLES::

            sage: from slabbe.language import RegularLanguage
            sage: alphabet = ['a', 'b']
            sage: trans = [(0, 1, 'a'), (1, 2, 'b'), (2, 3, 'b'), (3, 4, 'a')]
            sage: automaton = Automaton(trans, initial_states=[0], final_states=[4])
            sage: R = RegularLanguage(alphabet, automaton)
            sage: [list(R.words_of_length_iterator(i)) for i in range(6)]
            [[], [], [], [], [word: abba], []]
        """
        it = super(RegularLanguage, self).words_of_length_iterator(length)
        return [a for a in it if self._automaton(a)]

#####################
# Language generators
#####################
class LanguageGenerator(object):
    def ARP(self):
        r"""
        Return the Arnoux-Rauzy-Poincaré regular language.

            sage: from slabbe.language import languages
            sage: L = languages.ARP()
            sage: L
            Regular language over [1, 2, 3, 123, 132, 213, 231, 312, 321]
            defined by: Automaton with 7 states
            sage: [L.complexity(n) for n in range(4)]
            [1, 9, 57, 345]
        """
        alphabet = [1, 2, 3, 123, 132, 213, 231, 312, 321]
        automaton = self._ARP_automaton()
        return RegularLanguage(alphabet, automaton)

    def _ARP_automaton(self):
        r"""
        Return the automaton for the ARP language.

        EXAMPLES::

            sage: from slabbe.language import languages
            sage: A = languages._ARP_automaton()
            sage: A
            Automaton with 7 states

        TESTS::

            sage: A.process([1, 312, 1, 213])
            (True, 'H213')
            sage: A([1, 312, 1, 213])
            True
        """
        def H(i,j,k):
            return 'H{}{}{}'.format(i,j,k)
        def P(i,j,k):
            return int('{}{}{}'.format(i,j,k))
        def A(k):
            return int('{}'.format(k))
        D = 'Delta'
        states = [H(*p) for p in itertools.permutations((1,2,3))] + [D]
        autom = Automaton(initial_states=[D], final_states=states)
        for p in itertools.permutations((1,2,3)):
            i,j,k = p
            v = H(*p)
            autom.add_transition(v, H(k,i,j), P(k,i,j))
            autom.add_transition(v, H(j,k,i), P(j,k,i))
            autom.add_transition(v, H(k,j,i), P(k,j,i))
            autom.add_transition(v, D, A(i))
            autom.add_transition(v, v, A(j))
            autom.add_transition(D, v, P(i,j,k))
        for k in [1,2,3]:
            autom.add_transition(D, D, A(k))
        return autom

    def Brun(self):
        r"""
        Return the Brun regular language.

        EXAMPLES::

            sage: from slabbe.language import languages
            sage: L = languages.Brun()
            sage: L
            Regular language over [123, 132, 213, 231, 312, 321]
            defined by: Automaton with 6 states
            sage: [L.complexity(n) for n in range(4)]
            [1, 6, 18, 54]
            sage: list(L.words_of_length_iterator(2))
            [word: 123,123,
             word: 123,132,
             word: 123,312,
             word: 132,123,
             word: 132,132,
             word: 132,213,
             word: 213,213,
             word: 213,231,
             word: 213,321,
             word: 231,123,
             word: 231,213,
             word: 231,231,
             word: 312,231,
             word: 312,312,
             word: 312,321,
             word: 321,132,
             word: 321,312,
             word: 321,321]
        """
        alphabet = [123, 132, 213, 231, 312, 321]
        automaton = self._Brun_automaton()
        return RegularLanguage(alphabet, automaton)

    def _Brun_automaton(self):
        r"""
        Return the automaton for the Brun language.

        EXAMPLES::

            sage: from slabbe.language import languages
            sage: A = languages._Brun_automaton()
            sage: A
            Automaton with 6 states

        TESTS::

            sage: A([123, 123, 132, 213])
            True
            sage: A([123, 123, 132, 213, 123])
            False
        """
        def B(i,j,k):
            return int('{}{}{}'.format(i,j,k))
        states = [B(*p) for p in itertools.permutations((1,2,3))]
        autom = Automaton(initial_states=states, final_states=states)
        for p in itertools.permutations((1,2,3)):
            i,j,k = p
            autom.add_transition(B(*p), B(i,j,k), B(i,j,k))
            autom.add_transition(B(*p), B(i,k,j), B(i,k,j))
            autom.add_transition(B(*p), B(k,i,j), B(k,i,j))
        return autom

    def Selmer(self):
        r"""
        Return the Selmer regular language.

        EXAMPLES::

            sage: from slabbe.language import languages
            sage: L = languages.Selmer()
            sage: L
            Regular language over [123, 132, 213, 231, 312, 321]
            defined by: Automaton with 6 states
            sage: [L.complexity(n) for n in range(4)]
            [1, 6, 12, 24]
            sage: list(L.words_of_length_iterator(2))
            [word: 123,132,
             word: 123,312,
             word: 132,123,
             word: 132,213,
             word: 213,231,
             word: 213,321,
             word: 231,123,
             word: 231,213,
             word: 312,231,
             word: 312,321,
             word: 321,132,
             word: 321,312]
        """
        alphabet = [123, 132, 213, 231, 312, 321]
        automaton = self._Selmer_automaton()
        return RegularLanguage(alphabet, automaton)

    def _Selmer_automaton(self):
        r"""
        Return the automaton for the Selmer language.

        EXAMPLES::

            sage: from slabbe.language import languages
            sage: A = languages._Selmer_automaton()
            sage: A
            Automaton with 6 states

        TESTS::

            sage: A([123, 132, 213])
            True
            sage: A([123, 132, 213, 123])
            False
        """
        def S(i,j,k):
            return int('{}{}{}'.format(i,j,k))
        states = [S(*p) for p in itertools.permutations((1,2,3))]
        autom = Automaton(initial_states=states, final_states=states)
        for p in itertools.permutations((1,2,3)):
            i,j,k = p
            autom.add_transition(S(*p), S(i,k,j), S(i,k,j))
            autom.add_transition(S(*p), S(k,i,j), S(k,i,j))
        return autom

    def Cassaigne(self):
        r"""
        Return the Cassaigne regular language over the alphabet
        [11, 22, 122, 211, 121, 212].

        EXAMPLES::

            sage: from slabbe.language import languages
            sage: L = languages.Cassaigne()
            sage: L
            Regular language over [11, 22, 121, 122, 211, 212]
            defined by: Automaton with 1 state
            sage: [L.complexity(n) for n in range(4)]
            [1, 6, 36, 216]
        """
        alphabet = [11, 22, 122, 211, 121, 212]
        automaton = self._Cassaigne_automaton()
        return RegularLanguage(alphabet, automaton)

    def _Cassaigne_automaton(self):
        r"""
        Return the automaton for the Cassaigne language over the alphabet
        [11, 22, 122, 211, 121, 212].

        EXAMPLES::

            sage: from slabbe.language import languages
            sage: A = languages._Cassaigne_automaton()
            sage: A
            Automaton with 1 state
        """
        q = 0
        states = [q]
        autom = Automaton(initial_states=states, final_states=states)
        alphabet = [11, 22, 122, 211, 121, 212]
        for i in alphabet:
            autom.add_transition(q, q, i)
        return autom

languages = LanguageGenerator()



# -*- coding: utf-8 -*-
r"""
Multidimensional Continued Fraction Algorithms (Python code)

EXAMPLES::

    sage: from slabbe.mult_cont_frac import Brun
    sage: algo = Brun()

Drawing the natural extension::

    sage: fig = algo.natural_extension_plot(3000, norm_xyz=1, axis_off=True)
    sage: fig
    <Figure size 1200x200 with 4 Axes>
    sage: fig.savefig('a.png')  # not tested

Drawing the invariant measure::

    sage: fig = algo.invariant_measure_wireframe_plot(10^6, 50)
    sage: fig
    <Figure size 640x480 with 1 Axes>
    sage: fig.savefig('a.png')  # not tested

Word with given frequencies::

    sage: algo.s_adic_word((1,e,pi))
    word: 1232323123233231232332312323123232312323...

Construction of the same s-adic word from the substitutions and the coding
iterator::

    sage: from itertools import repeat
    sage: D = algo.substitutions()
    sage: it = algo.coding_iterator((1,e,pi))
    sage: words.s_adic(it, repeat(1), D)
    word: 1232323123233231232332312323123232312323...

AUTHORS:

 - Sébastien Labbé, Externalize Python only functions (pip install takes now 33s
   instead of 51s), August 2016
"""
#*****************************************************************************
#       Copyright (C) 2013-2016 Sébastien Labbé <slabqc@gmail.com>
#
#  Distributed under the terms of the GNU General Public License version 2 (GPLv2)
#
#  The full text of the GPLv2 is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************
from __future__ import absolute_import, print_function

from sage.structure.dynamic_class import dynamic_class
from slabbe import mult_cont_frac_pyx

PGF_COLORS = ["red", "green", "blue", "cyan", "brown", "gray", "orange", "pink",
"yellow", "black", "white", "darkgray", "lightgray",
"lime", "olive", "magenta", "purple", "teal", "violet"]

####################
# MCF Python methods
####################
class _MCFAlgorithm_methods(object):
    ######################
    # COMBINATORICS METHODS
    ######################
    def matrix_cocycle(self):
        r"""
        EXAMPLES::

            sage: from slabbe.mult_cont_frac import ARP
            sage: ARP().matrix_cocycle()
            Cocycle with 9 gens over Regular language over [1, 2, 3, 
            123, 132, 213, 231, 312, 321]
            defined by: Automaton with 7 states

        ::

            sage: from slabbe.mult_cont_frac import Sorted_Brun
            sage: Sorted_Brun().matrix_cocycle()
            Cocycle with 3 gens over Language of finite words over alphabet
            [1, 2, 3]

        ::

            sage: from slabbe.mult_cont_frac import Brun
            sage: Brun().matrix_cocycle()
            Cocycle with 6 gens over Regular language over 
            [123, 132, 213, 231, 312, 321]
            defined by: Automaton with 6 states
        """
        from .matrix_cocycle import cocycles
        try:
            f = getattr(cocycles, self.class_name())
        except AttributeError:
            msg = "Matrix cocyle not implemented for {}"
            msg = msg.format(self.class_name())
            raise NotImplementedError(msg)
        return f()

    def n_matrix(self, start, n_iterations):
        r"""
        Return the n-matrix associated to the direction v.

        INPUT:

        - ``start`` -- iterable of three real numbers
        - ``n_iterations`` - integer, number of iterations

        OUTPUT:

            matrix

        EXAMPLES::

            sage: from slabbe.mult_cont_frac import ARP
            sage: ARP().n_matrix((1,e,pi), 10)
            [ 31  40   7]
            [ 84 109  19]
            [ 97 126  22]
        """
        it = self.coding_iterator(start)
        L = [next(it) for _ in range(n_iterations)]
        cocycle = self.matrix_cocycle()
        return cocycle.word_to_matrix(L)

    def s_adic_word(self, v=None, n_iterations=100, nth_letter=1):
        r"""
        Return the s-adic word obtained from application of the MCF
        algorithm on the vector v.

        INPUT:

        - ``v`` - initial vector (default: ``None``), if None, then
          initial point is random
        - ``n_iterations`` - integer (default: ``100``), number of
          iterations
        - ``nth_letter`` - letter (default: ``1``)

        OUTPUT:

            word

        EXAMPLES::

            sage: from slabbe.mult_cont_frac import Brun, ARP, Cassaigne
            sage: Brun().s_adic_word((.414578,.571324,.65513))
            word: 1232312312323123123312323123123312323123...
            sage: Brun().s_adic_word((1,e,pi))
            word: 1232323123233231232332312323123232312323...
            sage: ARP().s_adic_word((1,e,pi))
            word: 1232323123233231232332312323123232312323...
            sage: Cassaigne().s_adic_word((1,e,pi))
            word: 2323213232323132323213232321323231323232...

        On integer entries::

            sage: w = Brun().s_adic_word((3,4,5))
            sage: w
            word: 123233123123
            sage: w.abelian_vector()
            [3, 4, 5]

        When the gcd is not one::

            sage: Brun().s_adic_word((2,4,10))
            word: 1233323312333233

        ::

            sage: from slabbe.mult_cont_frac import FullySubtractive
            sage: FullySubtractive().s_adic_word((1,2,5))
            Traceback (most recent call last):
            ...
            ValueError: On input=(1, 2, 5), algorithm Fully Subtractive
            loops on (1.0, 0.0, 3.0)

        ::

            sage: from slabbe.mult_cont_frac import Reverse
            sage: algo = Reverse()
            sage: algo.s_adic_word((18,1,1))
            word: 31111111111211111111
            sage: _.abelian_vector()
            [18, 1, 1]

        ::

            sage: Reverse().s_adic_word((3,1,1))
            Traceback (most recent call last):
            ...
            ValueError: On input=(3, 1, 1), algorithm Reverse reaches non
            integer entries (0.5, 0.5, 0.5)

        TESTS::

            sage: v = ARP().s_adic_word((1,e,pi))
            sage: w = Brun().s_adic_word((1,e,pi))
            sage: v.longest_common_prefix(w, 'finite').length()
            212
        """
        from sage.combinat.words.word_generators import words
        from sage.rings.integer_ring import ZZ
        if v is None:
            v = (random(), random(), random())
        D = self.substitutions()
        if all(a in ZZ for a in v):
            S = []
            it = self.cone_orbit_iterator(v)
            previousA = None
            for P,b in it:
                A,B = P.to_tuple()
                if not all(a in ZZ for a in A):
                    raise ValueError("On input={}, algorithm {} reaches"
                            " non integer entries {}".format(v, self.name(), A))
                if A == previousA:
                    break
                S.append(b)
                previousA = A
            (x,y,z) = A
            if x == 0 == y: 
                letter = 3
                the_gcd = ZZ(z)
            elif x == 0 == z:
                letter = 2
                the_gcd = ZZ(y)
            elif y == 0 == z:
                letter = 1
                the_gcd = ZZ(x)
            else:
                raise ValueError("On input={}, algorithm {} loops"
                                 " on {}".format(v, self.name(), A))
            return words.s_adic(S, [letter], D)**the_gcd

        else:
            it = self.coding_iterator(v)
            S = [next(it) for _ in range(n_iterations)]
            letter = nth_letter
            L = [letter]
            for key in reversed(S):
                letter = D[key](letter)[0]
                L.append(letter)
            L.pop()
            L.reverse()
            return words.s_adic(S, L, D)

    def discrepancy_statistics(self, length):
        r"""
        Return the discrepancy of words of given length.

        INPUT:

        - ``length`` -- integer

        OUTPUT:

            dict

        EXAMPLES::

            sage: from slabbe.mult_cont_frac import Brun
            sage: Brun().discrepancy_statistics(5)
            {[1, 1, 3]: 6/5,
             [1, 2, 2]: 4/5,
             [1, 3, 1]: 4/5,
             [2, 1, 2]: 3/5,
             [2, 2, 1]: 3/5,
             [3, 1, 1]: 4/5}
        """
        from .finite_word import discrepancy
        from sage.combinat.composition import Compositions
        D = {}
        for c in Compositions(length, length=3, min_part=1):
            w = self.s_adic_word(c)
            if c != w.abelian_vector(): 
                raise ValueError("c={} but vector is"
                      " {}".format(c,w.abelian_vector()))
            D[c] = discrepancy(w)
        return D

    def e_one_star_patch(self, v, n):
        r"""
        Return the n-th iterated patch of normal vector v.

        INPUT:

        - ``v`` -- vector, the normal vector
        - ``n`` -- integer

        EXAMPLES::

            sage: from slabbe.mult_cont_frac import FullySubtractive
            sage: FullySubtractive().e_one_star_patch((1,e,pi), 4)
            Patch of 21 faces
        """
        from sage.combinat.e_one_star import E1Star, Patch, Face
        from sage.misc.misc_c import prod
        if v is None:
            v = (random(), random(), random())
        it = self.coding_iterator(v)
        keys = [next(it) for _ in range(n)]
        D = self.dual_substitutions()
        L = prod(D[key] for key in reversed(keys))
        dual_sub = E1Star(L)
        cube = Patch([Face((1,0,0),1), Face((0,1,0),2), Face((0,0,1),3)])
        return dual_sub(cube)

    def _base_translation_vectors(self):
        r"""
        Return the base translation vectors associated to each elementary matrix

        EXAMPLES::

            sage: from slabbe.mult_cont_frac import FullySubtractive
            sage: algoFS = FullySubtractive()
            sage: algoFS._base_translation_vectors()
            {1: (1, 0, 0), 2: (0, 1, 0), 3: (0, 0, 1)}

        ::

            sage: from slabbe.mult_cont_frac import ARP
            sage: algoARP = ARP()
            sage: algoARP._base_translation_vectors()
            {1: (0, 1, 1),
             2: (1, 0, 1),
             3: (1, 1, 0),
             123: (1, 1, 0),
             132: (1, 0, 1),
             213: (1, 1, 0),
             231: (0, 1, 1),
             312: (1, 0, 1),
             321: (0, 1, 1)}
        """
        from sage.modules.free_module_element import vector
        def e(i):
            A = [0,0,0]
            A[i-1] = 1
            return vector(A)
        D = {}
        gens = self.matrix_cocycle().gens()
        for key,M in gens.items():
            Minv = M.inverse()
            v = sum(e(list(c).index(1)+1) for c in Minv.columns() if -1 in c)
            D[key] = v
        return D

    def translation_vectors(self, start, n_iterations):
        r"""
        Return the base translation vectors associated to each elementary matrix

        EXAMPLES::

            sage: from slabbe.mult_cont_frac import FullySubtractive, ARP, Brun
            sage: ARP().translation_vectors((1,e,pi), 5)
            [(1, 1, 0), (1, -1, 1), (-2, 1, 0), (1, 1, -1), (-3, 0, 1)]
            sage: Brun().translation_vectors((1,e,pi), 5)
            [(0, 1, 0), (1, 0, 0), (1, 0, 0), (-2, 1, 0), (0, -1, 1)]
            sage: FullySubtractive().translation_vectors((1,e,pi), 5)
            [(1, 0, 0), (1, 0, 0), (-2, 1, 0), (3, -1, 0), (-3, 0, 1)]
        """
        D = self._base_translation_vectors()
        it = self.coding_iterator(start)
        L = [next(it) for _ in range(n_iterations)]
        cocycle = self.matrix_cocycle()
        gens_inv = cocycle.gens_inverses()
        Minv = cocycle.identity_matrix()
        T = []
        for a in L:
            v = D[a] * Minv
            v.set_immutable()
            T.append(v)
            Minv = gens_inv[a] * Minv
        return T

    def discrete_plane_patches(self, start, n_iterations):
        r"""
        Return the patches subsets of a discrete plane using method from
        [JLP2016]_.

        INPUT:

        - ``start`` -- normal vector
        - ``n_iterations`` -- integer, number of iterations

        EXAMPLES:

        Partial sums for Brun forms a connected subset of Z^3::

            sage: from slabbe.mult_cont_frac import FullySubtractive, ARP, Brun
            sage: A = Brun().discrete_plane_patches((1,e,pi), 10)
            sage: _ = A.tikz().pdf()     # long time

        ::

            sage: A = FullySubtractive().discrete_plane_patches((1,e,pi), 10)
            sage: _ = A.tikz().pdf()     # not tested

        Result is not connected::

            sage: A = ARP().discrete_plane_patches((1,e,pi), 10)
            sage: _ = A.tikz().pdf()     # not tested

        REFERENCES:

        .. [JLP2016] Jamet, Damien, Nadia Lafrenière, et Xavier Provençal. « Generation
           of Digital Planes Using Generalized Continued-Fractions Algorithms
           ». In Discrete Geometry for Computer Imagery, édité par Nicolas
           Normand, Jeanpierre Guédon, et Florent Autrusseau, 45-56. Lecture
           Notes in Computer Science 9647. Springer International Publishing,
           2016. http://link.springer.com/chapter/10.1007/978-3-319-32360-2_4.
        """
        T = self.translation_vectors(start, n_iterations)
        zero = T[0].parent().zero()
        from sage.combinat.subset import Subsets
        S = Subsets(range(len(T)))
        V = [sum((T[s] for s in subset), zero) for subset in S]
        from slabbe import DiscreteSubset
        return DiscreteSubset.from_subset(V)

    ######################
    # DRAWINGS METHODS (python):
    ######################
    def invariant_measure_wireframe_plot(self, n_iterations, ndivs, norm=1):
        r"""
        Return a matplotlib graph of the invariant measure.

        INPUT:

        - ``n_iterations`` - integer, number of iterations
        - ``ndvis`` - integer, number of divisions per dimension
        - ``norm`` -- string (default: ``1``), either ``0`` or
          ``1``, the p-norm used for the orbit points

        EXAMPLES::

            sage: from slabbe.mult_cont_frac import Reverse, Brun
            sage: Reverse().invariant_measure_wireframe_plot(1000000, 80)
            <Figure size 640x480 with 1 Axes>
            sage: Brun().invariant_measure_wireframe_plot(1000000, 40, norm=1)
            <Figure size 640x480 with 1 Axes>

        """
        D = self._invariant_measure_dict(n_iterations, ndivs, norm=norm)
        the_mean = n_iterations / float(len(D))

        X = [[i for i in range(ndivs+1)] for j in range(ndivs+1)]
        Y = [[j for i in range(ndivs+1)] for j in range(ndivs+1)]
        Z = [[D.get((i,j),0)/the_mean for i in range(ndivs+1)] for j in range(ndivs+1)]
        import numpy
        Z = numpy.array(Z)
        
        from mpl_toolkits.mplot3d import axes3d
        import matplotlib.pyplot as plt
        fig = plt.figure()
        ax = fig.add_subplot(111, projection='3d')
        ax.plot_wireframe(X, Y, Z, rstride=1, cstride=1)

        ax.text(ndivs, 0, 0, "$(1,0,0)$", color='black', va='top', size=20)
        ax.text(0, ndivs, 0, "$(0,1,0)$", color='black', ha='left', size=20)
        ax.text(0, 0, 0, "$(0,0,1)$", color='black', ha='right', size=20)

        ax.set_xticklabels([])
        ax.set_yticklabels([])
        ax.set_zlabel('Frequency')

        title = "Density of an orbit for\n{} algorithm".format(self.name())
        ax.set_title(title)

        return fig

    def invariant_measure_contour_plot(self, n_iterations, ndivs, norm=1):
        r"""
        Return a matplotlib graph of the invariant measure.

        INPUT:

        - ``n_iterations`` - integer, number of iterations
        - ``ndvis`` - integer, number of divisions per dimension
        - ``norm`` -- string (default: ``1``), either ``0`` or
          ``1``, the p-norm used for the orbit points

        EXAMPLES::

            sage: from slabbe.mult_cont_frac import Reverse, Brun
            sage: Reverse().invariant_measure_contour_plot(1000000, 80)
            <Figure size 640x480 with 1 Axes>
            sage: Brun().invariant_measure_contour_plot(1000000, 40, norm=1)
            <Figure size 640x480 with 1 Axes>

        """
        D = self._invariant_measure_dict(n_iterations, ndivs, norm=norm)
        the_mean = n_iterations / float(len(D))
        S = sorted(D.values())
        V = [S[k]/the_mean for k in range(0, len(D), len(D)//10)]

        X = [[i for i in range(ndivs+1)] for j in range(ndivs+1)]
        Y = [[j for i in range(ndivs+1)] for j in range(ndivs+1)]
        Z = [[D.get((i,j),0)/the_mean for i in range(ndivs+1)] for j in range(ndivs+1)]
        
        import matplotlib.pyplot as plt
        fig = plt.figure()
        ax = fig.add_subplot(111)
        CS = plt.contour(X, Y, Z, V)
        plt.clabel(CS, inline=1, fontsize=10)

        title = "Density of an orbit for\n{} algorithm".format(self.name())
        ax.set_title(title)

        return fig

    def natural_extension_plot(self, n_iterations, norm_xyz=1,
            norm_uvw=1, axis_off=False):
        r"""
        INPUT:

        - ``n_iterations`` - integer, number of iterations
        - ``norm_xyz`` -- string (default: ``1``), either ``0`` or
          ``1``, the p-norm used for the orbit points
        - ``norm_uvw`` -- string (default: ``1``), either ``0`` or
          ``1``, the p-norm used for the dual orbit points
        - ``axis_off`` - boolean (default: ``False``), 

        EXAMPLES::

            sage: from slabbe.mult_cont_frac import Brun
            sage: Brun().natural_extension_plot(3000, norm_xyz=1, axis_off=True)
            <Figure size 1200x200 with 4 Axes>

        ::

            sage: from slabbe.mult_cont_frac import Sorted_ARP
            sage: Sorted_ARP().natural_extension_plot(1000)
            <Figure size 1200x200 with 4 Axes>
        """
        import matplotlib
        import matplotlib.pyplot as plt
        t = self._natural_extension_dict(n_iterations, norm_xyz=norm_xyz,
                norm_uvw=norm_uvw)
        domain_left, image_left, domain_right, image_right = t
        c = dict(zip(domain_left.keys(), ['b','r','g','c','m','y','k']))

        def create_sub_plot(axx, D, title, norm):
            for key, value in D.items():
                value = D[key]
                X,Y = zip(*value)
                A = axx.plot(X, Y, 'o', markersize=2, color=c[key], label=key)
            axx.legend(markerscale=3,fontsize="xx-small")
            axx.set_title(title)

        fig, (ax,bx,cx,dx) = plt.subplots(1,4,sharex=True,sharey=True)
        fig.set_figheight(2)
        fig.set_figwidth(12)
        if axis_off:
            ax.set_axis_off()
            bx.set_axis_off()
            cx.set_axis_off()
            dx.set_axis_off()

        create_sub_plot(ax, domain_left,  "Algo IN",    norm=norm_xyz)
        create_sub_plot(bx, domain_right,    "NatExt IN",  norm=norm_uvw)
        create_sub_plot(cx, image_left, "Algo OUT",   norm=norm_xyz)
        create_sub_plot(dx, image_right,   "NatExt OUT", norm=norm_uvw)

        title = "Algo=%s, nbiter=%s" % (self.name(), n_iterations)
        fig.suptitle(title)

        return fig

    def natural_extension_tikz(self, n_iterations, norm_xyz=1,
            norm_uvw=1, marksize=0.2, legend_marksize=2, 
            group_size="4 by 1"):
        r"""

        INPUT:

        - ``n_iterations`` -- integer, number of iterations
        - ``norm_xyz`` -- string (default: ``1``), either ``0`` or
          ``1``, the p-norm used for the orbit points
        - ``norm_uvw`` -- string (default: ``1``), either ``0`` or
          ``1``, the p-norm used for the dual orbit points
        - ``marksize`` -- tikz marksize (default:``0.2``)
        - ``legend_marksize`` -- tikz legend marksize (default:``2``)
        - ``group_size`` -- string (default:``"4 by 1"``)

        EXAMPLES::

            sage: from slabbe.mult_cont_frac import Brun
            sage: s = Brun().natural_extension_tikz(1000)
            sage: s
            \documentclass[tikz]{standalone}
            \usepackage{pgfplots}
            \usetikzlibrary{pgfplots.groupplots}
            \begin{document}
            \begin{tikzpicture}[scale=.7]
            \begin{groupplot}
            [group style={group size=4 by 1},
            height=7cm,width=8cm,
            xmin=-1.1,xmax=1.1,ymin=-.6,ymax=1.20,
            ...
            ... 4184 lines not printed (17... characters in total) ...
            ...
            \draw[draw=none] (group c2r1.center) --
            node {$\to$}     (group c3r1.center);
            \draw[draw=none] (group c3r1.center) --
            node {$\times$}  (group c4r1.center);
            \end{tikzpicture}
            \end{document}

        ::

            sage: from sage.misc.temporary_file import tmp_filename
            sage: filename = tmp_filename('temp','.pdf')
            sage: _ = s.pdf(filename)
        """
        t = self._natural_extension_dict(n_iterations, norm_xyz=norm_xyz,
                norm_uvw=norm_uvw)
        domain_left, image_left, domain_right, image_right = t
        sqrt3 = 1.73205080756888
        r = 1.08
        color_dict = dict(zip(domain_left.keys(), PGF_COLORS))

        lines = []
        lines.append(r"\begin{tikzpicture}[scale=.7]")
        lines.append(r"\begin{groupplot}")
        lines.append(r"[group style={group size=%s}," % group_size)
        lines.append(r"height=7cm,width=8cm,")
        lines.append(r"xmin=-1.1,xmax=1.1,ymin=-.6,ymax=1.20,")
        lines.append(r"hide axis]")
        datas = [domain_left, domain_right, image_left, image_right]
        labels = [r"$\mathbf{x}_n$",r"$\mathbf{a}_n$",
                  r"$\mathbf{x}_{n+1}$",r"$\mathbf{a}_{n+1}$"]
        for data,label in zip(datas, labels):
            lines.append(r"\nextgroupplot")
            lines.append(r"\draw[dashed] ")
            lines.append(r"(axis cs:%s, %s)" % (-r*sqrt3/2,r*-.5))
            lines.append(r" node[left] {$\mathbf{e}_1$} -- ")
            lines.append("(axis cs:%s, %s)" % (r*sqrt3/2,r*-.5))
            lines.append(r" node[right] {$\mathbf{e}_2$} -- ")
            lines.append("(axis cs:%s, %s)" % (0, r))
            lines.append(r" node[above] {$\mathbf{e}_3$} -- cycle;")
            lines.append(r"\node at (axis cs:%s, %s) {%s};" % (-.5,.8,label))
            for key,value in data.items():
                lines.append(r"\addplot+[")
                lines.append(r"legend image post style={mark size=%s}," % legend_marksize)
                lines.append(r"only marks,mark=*,")
                lines.append(r"mark size=%s," % marksize)
                lines.append(r"mark options={color=%s}] " % color_dict[key])
                lines.append(r"coordinates {%s};" % '\n'.join(map(str, value)))
                lines.append(r"\addlegendentry{%s}" % key)
        lines.append(r"\end{groupplot}")
        if group_size == "4 by 1":
            lines.append(r"\draw[draw=none] (group c1r1.center) -- ")
            lines.append(r"node {$\times$}  (group c2r1.center);")
            lines.append(r"\draw[draw=none] (group c2r1.center) -- ")
            lines.append(r"node {$\to$}     (group c3r1.center);")
            lines.append(r"\draw[draw=none] (group c3r1.center) -- ")
            lines.append(r"node {$\times$}  (group c4r1.center);")
        lines.append(r"\end{tikzpicture}")
        from slabbe import TikzPicture
        return TikzPicture('\n'.join(lines), usepackage=['pgfplots'],
                usetikzlibrary=['pgfplots.groupplots'])

    def natural_extension_part_tikz(self, n_iterations, part=3, 
                                    norm_xyz=1, norm_uvw=1,
                                    marksize='1pt', limit_nb_points=None,
                                    verbose=False):
        r"""
        Return a pgfplots or some part of an orbit in the natural
        extension.

        INPUT:

        - ``n_iterations`` - integer, number of iterations
        - ``part`` - integer, taking value 0, 1, 2 or 3
        - ``norm_xyz`` -- string (default: ``1``), either ``0`` or
          ``1``, the p-norm used for the orbit points
        - ``norm_uvw`` -- string (default: ``1``), either ``0`` or
          ``1``, the p-norm used for the dual orbit points
        - ``marksize`` -- string (default: ``'1pt'``), pgfplots mark size value
        - ``limit_nb_points`` -- None or integer (default: ``None``), limit
          number of points per patch
        - ``verbose`` -- string (default: ``False``)

        EXAMPLES::

            sage: from slabbe.mult_cont_frac import ARP
            sage: s = ARP().natural_extension_part_tikz(1000, part=3)
            sage: view(s, tightpage=True)    # not tested

        also ::

            sage: s = ARP().natural_extension_part_tikz(1000, part=3, marksize='.2pt', limit_nb_points=1200, verbose=True)
            Taking ... points for key ...
            Taking ... points for key ...
            Taking ... points for key ...
            Taking ... points for key ...
            Taking ... points for key ...
            Taking ... points for key ...
            Taking ... points for key ...
            Taking ... points for key ...
            Taking ... points for key ...
        """
        t = self._natural_extension_dict(n_iterations, norm_xyz=norm_xyz,
                norm_uvw=norm_uvw)
        domain_left, image_left, domain_right, image_right = t
        sqrt3 = 1.73205080756888
        r = 1.08
        color_dict = dict(zip(domain_left.keys(), PGF_COLORS))
        data = t[part]

        s = ''
        s += "\\begin{tikzpicture}[scale=.7]\n"
        s += ("\\begin{axis}[height=7cm,width=8cm,\n"
               "xmin=-1.1,xmax=1.1,ymin=-.6,ymax=1.20,\n"
               "hide axis]\n")
        s += ("\\draw[dashed] \n"
              "(axis cs:%s, %s)" % (-r*sqrt3/2,r*-.5) +
              " node[left] {$\\mathbf{e}_1$} -- \n"
              "(axis cs:%s, %s)" % (r*sqrt3/2,r*-.5)  +
              " node[right] {$\\mathbf{e}_2$} -- \n"
              "(axis cs:%s, %s)" % (0, r)             +
              " node[above] {$\\mathbf{e}_3$} -- cycle;\n")
        for key,value in data.items():
            if limit_nb_points and len(value) > limit_nb_points:
                if verbose:
                    print("Taking only {} points instead of {} for key {}".format(
                            limit_nb_points, len(value), key))
                value = value[:limit_nb_points]
            elif verbose:
                print("Taking {} points for key {}".format(len(value),
                    key))

            s += "\\addplot+[only marks,mark=*,mark options={color=%s}," % color_dict[key]
            s += "mark size=%s]\n" % marksize
            s += "coordinates {\n"
            s += '\n'.join(map(str, value))
            s += "};\n" 
            s += "\\addlegendentry{%s}\n " % key
        s += "\\end{axis}\n"
        s += "\\end{tikzpicture}\n"
        from slabbe import TikzPicture
        return TikzPicture(s, usepackage=['pgfplots'])

    def natural_extension_part_png(self, n_iterations, draw,
                                    norm_xyz=1, norm_uvw=1,
                                    xrange=(-.866, .866),
                                    yrange=(-.5, 1.),
                                    urange=(-.866, .866),
                                    vrange=(-.5, 1.),
                                    color_dict=None,
                                    branch_order=None,
                                    ndivs=1024,
                                    verbose=False):
        r"""
        Return a png or some part of an orbit in the natural extension.

        INPUT:

        - ``n_iterations`` - integer, number of iterations
        - ``draw`` -- string (default: ``'image_right'``), possible values
          are:

          - ``'domain_left'`` - use x and y ranges
          - ``'domain_right'`` - use u and v ranges
          - ``'image_left'`` - use x and y ranges
          - ``'image_right'`` - use u and v ranges

        - ``norm_xyz`` -- string (default: ``1``), either ``0`` or
          ``1``, the p-norm used for the orbit points
        - ``norm_uvw`` -- string (default: ``1``), either ``0`` or
          ``1``, the p-norm used for the dual orbit points
        - ``xrange`` -- tuple (default: ``(-.866, .866)``), interval of
          values for x
        - ``yrange`` -- tuple (default: ``(-.5, 1.)``), interval of
          values for y
        - ``urange`` -- tuple (default: ``(-.866, .866)``), interval of
          values for u
        - ``vrange`` -- tuple (default: ``(-.5, 1.)``), interval of
          values for v
        - ``color_dict`` -- dict (default: ``None``), dict from branches
          int to color (as RGB tuples)
        - ``branch_order`` -- list (default: ``None``), list of branches int
        - ``ndivs`` -- int (default: ``1024``), number of pixels
        - ``verbose`` -- string (default: ``False``)

        BENCHMARK:

        A minute (1min 13s) for a picture with 10^7 points::

            sage: from slabbe.mult_cont_frac import ARP
            sage: c = {}
            sage: c[1] = c[2] = c[3] = [0,0,0]
            sage: c[123] = c[132] = c[231] = c[213] = c[312] = c[321] = [255,0,0]
            sage: b = [1,2,3,123,132,213,231,312,321]
            sage: P = ARP().natural_extension_part_png(10^7, draw='image_right',  # not tested
            ....:   branch_order=b, color_dict=c, urange=(-.6,.6), vrange=(-.6,.6))  # not tested

        Half a minute (27s) for a picture zoomed in the orbit of 10^8
        points::

            sage: P = ARP().natural_extension_part_png(10^8, draw='image_right', # not tested
            ....:   branch_order=b, color_dict=c, urange=(.2,.3), vrange=(.2,.3))   # not tested

        EXAMPLES::

            sage: c = {}
            sage: c[1] = c[2] = c[3] = [0,0,0]
            sage: c[123] = c[132] = c[231] = c[213] = c[312] = c[321] = [255,0,0]
            sage: b = [1,2,3,123,132,213,231,312,321]
            sage: opt = dict(urange=(-.6,.6), vrange=(-.6,.6), color_dict=c, branch_order=b)
            sage: P = ARP().natural_extension_part_png(10^5, draw='domain_left', **opt)
            sage: P = ARP().natural_extension_part_png(10^5, draw='domain_right', **opt)
            sage: P = ARP().natural_extension_part_png(10^5, draw='image_left', **opt)
            sage: P = ARP().natural_extension_part_png(10^5, draw='image_right', **opt)
            sage: P.show() # not tested

        """
        L = self.simplex_orbit_filtered_list(n_iterations=n_iterations,
            norm_xyz=norm_xyz, norm_uvw=norm_uvw,
            xmin=xrange[0], xmax=xrange[1],
            ymin=yrange[0], ymax=yrange[1],
            umin=urange[0], umax=urange[1],
            vmin=vrange[0], vmax=vrange[1],
            ndivs=ndivs)
        if branch_order is None:
            branch_order = []
            raise NotImplementedError
        if color_dict is None:
            from random import randint
            color_dict = {}
            for key in branch_order:
                color_dict[key] = [randint(0,255),randint(0,255),randint(0,255)]

        #http://stackoverflow.com/questions/434583/what-is-the-fastest-way-to-draw-an-image-from-discrete-pixel-values-in-python
        import numpy as np

        # Create a 1024x1024x3 array of 8 bit unsigned integers
        data = np.zeros( (ndivs,ndivs,3), dtype=np.uint8 )
        data += 255   # white as default color

        if draw.startswith('domain'):
            L.sort(key=lambda a:branch_order.index(a[5]))
        elif draw.startswith('image'):
            L.sort(key=lambda a:branch_order.index(a[4]))
        else:
            raise ValueError("Unkown value for draw(={})".format(draw))

        if draw == 'domain_left':
            for x,y,u,v,prev_br,next_br in L:
                data[y,x] = color_dict[next_br]
        elif draw == 'domain_right':
            for x,y,u,v,prev_br,next_br in L:
                data[v,u] = color_dict[next_br]
        elif draw == 'image_left':
            for x,y,u,v,prev_br,next_br in L:
                data[y,x] = color_dict[prev_br]
        elif draw == 'image_right':
            for x,y,u,v,prev_br,next_br in L:
                data[v,u] = color_dict[prev_br]
        else:
            raise ValueError("Unkown value for draw(={})".format(draw))

        from PIL import Image
        img = Image.fromarray(data)
        #img.show()                      # View in default viewer
        return img

    def measure_evaluation(self, n_iterations, draw,
                                norm_xyz=1, norm_uvw=1,
                                xrange=(-.866, .866),
                                yrange=(-.5, 1.),
                                urange=(-.866, .866),
                                vrange=(-.5, 1.),
                                ndivs=1024,
                                verbose=False):
        r"""
        Return the measure of a box according to an orbit.

        INPUT:

        - ``n_iterations`` - integer, number of iterations
        - ``draw`` -- string (default: ``'image_right'``), possible values
          are:

          - ``'domain_left'`` - use x and y ranges
          - ``'domain_right'`` - use u and v ranges
          - ``'image_left'`` - use x and y ranges
          - ``'image_right'`` - use u and v ranges

        - ``norm_xyz`` -- string (default: ``1``), either ``0`` or
          ``1``, the p-norm used for the orbit points
        - ``norm_uvw`` -- string (default: ``1``), either ``0`` or
          ``1``, the p-norm used for the dual orbit points
        - ``xrange`` -- tuple (default: ``(-.866, .866)``), interval of
          values for x
        - ``yrange`` -- tuple (default: ``(-.5, 1.)``), interval of
          values for y
        - ``urange`` -- tuple (default: ``(-.866, .866)``), interval of
          values for u
        - ``vrange`` -- tuple (default: ``(-.5, 1.)``), interval of
          values for v
        - ``ndivs`` -- int (default: ``1024``), number of pixels
        - ``verbose`` -- string (default: ``False``)

        BENCHMARK::

            sage: from slabbe.mult_cont_frac import ARP
            sage: opt = dict(urange=(-.15,.25), vrange=(-.05,.05))
            sage: ARP().measure_evaluation(10^8, draw='right', ndivs=100, **opt) # optional long
            0.435...
            sage: ARP().measure_evaluation(10^8, draw='right', ndivs=1000, **opt) # optional long
            0.357...
            sage: ARP().measure_evaluation(10^8, draw='right', ndivs=2000, **opt) # optional long
            0.293...
            sage: ARP().measure_evaluation(10^8, draw='right', ndivs=4000, **opt) # optional long
            0.177...

        """
        L = self.simplex_orbit_filtered_list(n_iterations=n_iterations,
            norm_xyz=norm_xyz, norm_uvw=norm_uvw,
            xmin=xrange[0], xmax=xrange[1],
            ymin=yrange[0], ymax=yrange[1],
            umin=urange[0], umax=urange[1],
            vmin=vrange[0], vmax=vrange[1],
            ndivs=ndivs)

        if draw.endswith('left'):
            S = set((p[0],p[1]) for p in L)
        elif draw.endswith('right'):
            S = set((p[2],p[3]) for p in L)
        else:
            raise ValueError("Unkown value for draw(={})".format(draw))

        if verbose:
            print("nombre diterations dans la fenetre : ", len(L))
            print("{} pixels touchés parmi limage {}^2 ".format(len(S),
                ndivs))

        print(float(len(S) / ndivs**2))

########################
# Dynamic class creation
########################
def _dynamic_MCF_class(base):
    r"""
    Return the MCF algorithm associated to some Cython base class including all
    the Python methods.

    This is where the fusion of cython methods with python methods is done with
    the use of ``dynamic_class``.

    INPUT:

    - ``base`` -- MCF algorithm base class

    OUTPUT:

        class

    EXAMPLES::

        sage: from slabbe.mult_cont_frac_pyx import Brun
        sage: from slabbe.mult_cont_frac import _dynamic_MCF_class
        sage: cls = _dynamic_MCF_class(Brun)
        sage: algo = cls()
        sage: algo
        Brun 3-dimensional continued fraction algorithm
        sage: algo.matrix_cocycle()
        Cocycle with 6 gens over Regular language over [123, 132, 213, 231, 312, 321]
        defined by: Automaton with 6 states
    """
    class_name = base().class_name()
    return dynamic_class(class_name, (base, _MCFAlgorithm_methods))

################
# MCF Algorithms
################
def Brun(dim=3):
    r"""
    EXAMPLES::

        sage: from slabbe.mult_cont_frac import Brun
        sage: Brun()
        Brun 3-dimensional continued fraction algorithm
    """
    return _dynamic_MCF_class(mult_cont_frac_pyx.Brun)(dim)
def Reverse(dim=3):
    r"""
    EXAMPLES::

        sage: from slabbe.mult_cont_frac import Reverse
        sage: Reverse()
        Reverse 3-dimensional continued fraction algorithm
    """
    return _dynamic_MCF_class(mult_cont_frac_pyx.Reverse)(dim)
def ARP(dim=3):
    r"""
    EXAMPLES::

        sage: from slabbe.mult_cont_frac import ARP
        sage: ARP()
        Arnoux-Rauzy-Poincar\'e 3-dimensional continued fraction algorithm
    """
    return _dynamic_MCF_class(mult_cont_frac_pyx.ARP)(dim)
def ArnouxRauzy(dim=3):
    r"""
    EXAMPLES::

        sage: from slabbe.mult_cont_frac import ArnouxRauzy
        sage: ArnouxRauzy()
        ArnouxRauzy 3-dimensional continued fraction algorithm
    """
    return _dynamic_MCF_class(mult_cont_frac_pyx.ArnouxRauzy)(dim)
def Poincare(dim=3):
    r"""
    EXAMPLES::

        sage: from slabbe.mult_cont_frac import Poincare
        sage: Poincare()
        Poincar\'e 3-dimensional continued fraction algorithm
    """
    return _dynamic_MCF_class(mult_cont_frac_pyx.Poincare)(dim)
def Selmer(dim=3):
    r"""
    EXAMPLES::

        sage: from slabbe.mult_cont_frac import Selmer
        sage: Selmer()
        Selmer 3-dimensional continued fraction algorithm
    """
    return _dynamic_MCF_class(mult_cont_frac_pyx.Selmer)(dim)
def FullySubtractive(dim=3):
    r"""
    EXAMPLES::

        sage: from slabbe.mult_cont_frac import FullySubtractive
        sage: FullySubtractive()
        Fully Subtractive 3-dimensional continued fraction algorithm
    """
    return _dynamic_MCF_class(mult_cont_frac_pyx.FullySubtractive)(dim)
def Cassaigne(dim=3):
    r"""
    EXAMPLES::

        sage: from slabbe.mult_cont_frac import Cassaigne
        sage: Cassaigne()
        Cassaigne 3-dimensional continued fraction algorithm
    """
    return _dynamic_MCF_class(mult_cont_frac_pyx.Cassaigne)(dim)
def Sorted_Brun():
    r"""
    EXAMPLES::

        sage: from slabbe.mult_cont_frac import Sorted_Brun
        sage: Sorted_Brun()
        Sorted_Brun 3-dimensional continued fraction algorithm
    """
    return _dynamic_MCF_class(mult_cont_frac_pyx.Sorted_Brun)()
def Sorted_BrunMulti():
    r"""
    EXAMPLES::

        sage: from slabbe.mult_cont_frac import Sorted_BrunMulti
        sage: Sorted_BrunMulti()
        Sorted_BrunMulti 3-dimensional continued fraction algorithm
    """
    return _dynamic_MCF_class(mult_cont_frac_pyx.Sorted_BrunMulti)()
def Sorted_Selmer():
    r"""
    EXAMPLES::

        sage: from slabbe.mult_cont_frac import Sorted_Selmer
        sage: Sorted_Selmer()
        Sorted_Selmer 3-dimensional continued fraction algorithm
    """
    return _dynamic_MCF_class(mult_cont_frac_pyx.Sorted_Selmer)()
def Sorted_FullySubtractive():
    r"""
    EXAMPLES::

        sage: from slabbe.mult_cont_frac import Sorted_FullySubtractive
        sage: Sorted_FullySubtractive()
        Sorted_FullySubtractive 3-dimensional continued fraction algorithm
    """
    return _dynamic_MCF_class(mult_cont_frac_pyx.Sorted_FullySubtractive)()
def Sorted_ArnouxRauzy():
    r"""
    EXAMPLES::

        sage: from slabbe.mult_cont_frac import Sorted_ArnouxRauzy
        sage: Sorted_ArnouxRauzy()
        Sorted_ArnouxRauzy 3-dimensional continued fraction algorithm
    """
    return _dynamic_MCF_class(mult_cont_frac_pyx.Sorted_ArnouxRauzy)()
def Sorted_ArnouxRauzyMulti():
    r"""
    EXAMPLES::

        sage: from slabbe.mult_cont_frac import Sorted_ArnouxRauzyMulti
        sage: Sorted_ArnouxRauzyMulti()
        Sorted_ArnouxRauzyMulti 3-dimensional continued fraction algorithm
    """
    return _dynamic_MCF_class(mult_cont_frac_pyx.Sorted_ArnouxRauzyMulti)()
def Sorted_ARP():
    r"""
    EXAMPLES::

        sage: from slabbe.mult_cont_frac import Sorted_ARP
        sage: Sorted_ARP()
        Sorted_ARP 3-dimensional continued fraction algorithm
    """
    return _dynamic_MCF_class(mult_cont_frac_pyx.Sorted_ARP)()
def Sorted_ARPMulti():
    r"""
    EXAMPLES::

        sage: from slabbe.mult_cont_frac import Sorted_ARPMulti
        sage: Sorted_ARPMulti()
        Sorted_ARPMulti 3-dimensional continued fraction algorithm
    """
    return _dynamic_MCF_class(mult_cont_frac_pyx.Sorted_ARPMulti)()
def Sorted_Poincare():
    r"""
    EXAMPLES::

        sage: from slabbe.mult_cont_frac import Sorted_Poincare
        sage: Sorted_Poincare()
        Sorted_Poincare 3-dimensional continued fraction algorithm
    """
    return _dynamic_MCF_class(mult_cont_frac_pyx.Sorted_Poincare)()
def Sorted_ARrevert():
    r"""
    EXAMPLES::

        sage: from slabbe.mult_cont_frac import Sorted_ARrevert
        sage: Sorted_ARrevert()
        Sorted_ARrevert 3-dimensional continued fraction algorithm
    """
    return _dynamic_MCF_class(mult_cont_frac_pyx.Sorted_ARrevert)()
def Sorted_ARrevertMulti():
    r"""
    EXAMPLES::

        sage: from slabbe.mult_cont_frac import Sorted_ARrevertMulti
        sage: Sorted_ARrevertMulti()
        Sorted_ARrevertMulti 3-dimensional continued fraction algorithm
    """
    return _dynamic_MCF_class(mult_cont_frac_pyx.Sorted_ARrevertMulti)()
def Sorted_ARMonteil():
    r"""
    EXAMPLES::

        sage: from slabbe.mult_cont_frac import Sorted_ARMonteil
        sage: Sorted_ARMonteil()
        Sorted_ARMonteil 3-dimensional continued fraction algorithm
    """
    return _dynamic_MCF_class(mult_cont_frac_pyx.Sorted_ARMonteil)()
def Sorted_Delaunay():
    r"""
    EXAMPLES::

        sage: from slabbe.mult_cont_frac import Sorted_Delaunay
        sage: Sorted_Delaunay()
        Sorted_Delaunay 3-dimensional continued fraction algorithm
    """
    return _dynamic_MCF_class(mult_cont_frac_pyx.Sorted_Delaunay)()
def JacobiPerron():
    r"""
    EXAMPLES::

        sage: from slabbe.mult_cont_frac import JacobiPerron
        sage: JacobiPerron()
        JacobiPerron 3-dimensional continued fraction algorithm
    """
    return _dynamic_MCF_class(mult_cont_frac_pyx.JacobiPerron)()
def JacobiPerronAdditif():
    r"""
    EXAMPLES::

        sage: from slabbe.mult_cont_frac import JacobiPerronAdditif
        sage: JacobiPerronAdditif()
        JacobiPerronAdditif 3-dimensional continued fraction algorithm
    """
    return _dynamic_MCF_class(mult_cont_frac_pyx.JacobiPerronAdditif)()
def JacobiPerronAdditifv2():
    r"""
    EXAMPLES::

        sage: from slabbe.mult_cont_frac import JacobiPerronAdditifv2
        sage: JacobiPerronAdditifv2()
        JacobiPerronAdditifv2 3-dimensional continued fraction algorithm
    """
    return _dynamic_MCF_class(mult_cont_frac_pyx.JacobiPerronAdditifv2)()



from __future__ import absolute_import, print_function

# Eventually, toward support of Python 3
#from __future__ import division, absolute_import, print_function, unicode_literals

# For sphinx to work, we first need to import the sage library
from sage.all_cmdline import *

from sage.misc.latex import latex
latex.add_to_preamble('\\usepackage{tikz}')
latex.add_to_preamble('\\usepackage{pgfplots}')
latex.add_to_preamble('\\usetikzlibrary{pgfplots.groupplots}')

from .discrete_subset import DiscreteSubset, DiscreteBox, DiscreteTube, Intersection
from .billiard import BilliardCube
from .discrete_plane import DiscretePlane, DiscreteLine, DiscreteHyperplane
from .christoffel_graph import ChristoffelGraph
from .bispecial_extension_type import ExtensionType, ExtensionType1to1, ExtensionTypeLong
from .double_square_tile import DoubleSquare, christoffel_tile
from .fruit import Fruit, Banana, Strawberry
from .joyal_bijection import Endofunctions, Endofunction, DoubleRootedTree
from .bond_percolation import (BondPercolationSamples, 
                             BondPercolationSample, 
                             PercolationProbability)
from .tikz_picture import TikzPicture, StandaloneTex
from .ddim_sturmian_configuration import dSturmianConfiguration

from .matrices import M3to2, M2to3, M4to2, M4to3

from .substitution_2d import Substitution2d
from .wang_tiles import WangTiling, WangTileSolver, WangTileSet
from .polyhedron_partition import PolyhedronPartition
from .polyhedron_exchange_transformation import PolyhedronExchangeTransformation
from .coding_of_PETs import PETsCoding

from .partial_injection import (number_of_partial_injection,
                                random_partial_injection,
                                random_cyclically_reduced_stallings_graph)

from .write_to_file import write_str_to_file

from .jeandel_rao import random_jeandel_rao_tiling_rectangle

# BUG (sometimes, cython code does not work properly)
try:
    from .kolakoski_word import KolakoskiWord
except ImportError:
    print("There was an error while importing KolakoskiWord cython module")
    print("Maybe that's because the pwd contains the slabbe folder?")
    print("We ignore the problem but this module won't work.")

# for doctext to work, we import convex_boundary
# from .discrete_subset import convex_boundary

# do not import module names just the above stuff
#__all__ = []



# -*- coding: utf-8 -*-
r"""
Word morphisms methods and iterators

EXAMPLES::

    sage: from slabbe.word_morphisms import iter_primitive_marked_classP_morphisms
    sage: F = FiniteWords('ab')
    sage: it = iter_primitive_marked_classP_morphisms(F, 4)
    sage: list(it)
    [WordMorphism: a->aba, b->a,
     WordMorphism: a->bab, b->a,
     WordMorphism: a->b, b->aba,
     WordMorphism: a->b, b->bab,
     WordMorphism: a->abb, b->a,
     WordMorphism: a->ab, b->aa,
     WordMorphism: a->bb, b->ba,
     WordMorphism: a->b, b->baa]
"""
#*****************************************************************************
#       Copyright (C) 2016 Sebastien Labbe <slabqc@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************
from __future__ import absolute_import, print_function
from sage.combinat.integer_lists.invlex import IntegerListsLex
from sage.combinat.words.morphism import WordMorphism
from sage.combinat.words.words import FiniteWords
import itertools

def iter_palindromes(words, length):
    r"""
    EXAMPLES::

        sage: from slabbe.word_morphisms import iter_palindromes
        sage: list(iter_palindromes(Words('ab'), 2))
        [word: aa, word: bb]
        sage: list(iter_palindromes(Words('ab'), 3))
        [word: aaa, word: aba, word: bab, word: bbb]
        sage: list(iter_palindromes(Words('ab'), 4))
        [word: aaaa, word: abba, word: baab, word: bbbb]
        sage: list(iter_palindromes(Words('ab'), 5))
        [word: aaaaa,
         word: aabaa,
         word: ababa,
         word: abbba,
         word: baaab,
         word: babab,
         word: bbabb,
         word: bbbbb]
    """
    if length % 2 == 0:
        for w in words.iterate_by_length(length // 2):
            yield w * w.reversal()
    else:
        for w in words.iterate_by_length(length // 2):
            for a in words.iterate_by_length(1):
                yield w * a * w.reversal()

def iter_conjugate_classP(words, n):
    r"""
    EXAMPLES::

        sage: from slabbe.word_morphisms import iter_conjugate_classP
        sage: F = FiniteWords('ab')
        sage: list(iter_conjugate_classP(F, 2))
        [WordMorphism: a->a, b->a,
         WordMorphism: a->a, b->b,
         WordMorphism: a->b, b->a,
         WordMorphism: a->b, b->b]
        sage: list(iter_conjugate_classP(F, 3))
        [WordMorphism: a->aa, b->a,
         WordMorphism: a->aa, b->b,
         WordMorphism: a->bb, b->a,
         WordMorphism: a->bb, b->b,
         WordMorphism: a->a, b->aa,
         WordMorphism: a->a, b->bb,
         WordMorphism: a->b, b->aa,
         WordMorphism: a->b, b->bb,
         WordMorphism: a->ba, b->b,
         WordMorphism: a->ab, b->a,
         WordMorphism: a->b, b->ba,
         WordMorphism: a->a, b->ab]
    """
    alphabet = words.alphabet()
    length = alphabet.cardinality()
    # images are palindromes
    for sizes in IntegerListsLex(n=n, length=length, min_part=1):
        L = [iter_palindromes(words, size) for size in sizes]
        for pals in itertools.product(*L):
            d = dict(zip(alphabet, pals))
            yield WordMorphism(d, codomain=words)
    # images are one common letter + palindrome
    for sizes in IntegerListsLex(n=n-length, length=length, min_part=0):
        L = [iter_palindromes(words, size) for size in sizes]
        for pals in itertools.product(*L):
            for b in alphabet:
                d = {a:words([b])*p for a,p in zip(alphabet, pals)}
                if all(w.is_palindrome() for w in d.values()):
                    # already yielded above
                    continue
                yield WordMorphism(d, codomain=words)

def iter_pisot_irreductible(d=3, arg=None):
    r"""
    Return an iterator over Pisot irreductible substitutions

    INPUT:

    - ``d`` -- size of alphabet, [0,1,...,d-1]

    - "arg" -- (optional, default: None) It can be one of the
      following :

      * "None" -- then the method iterates through all morphisms.

      * tuple (a, b) of two integers  - It specifies the range
        "range(a, b)" of values to consider for the sum of the length

    EXAMPLES::

        sage: from slabbe.word_morphisms import iter_pisot_irreductible
        sage: it = iter_pisot_irreductible(3)
        sage: for _ in range(4): next(it)
        WordMorphism: 0->01, 1->2, 2->0
        WordMorphism: 0->02, 1->0, 2->1
        WordMorphism: 0->10, 1->2, 2->0
        WordMorphism: 0->12, 1->0, 2->1

    Pour linstant, avec le tuple, il y a un bogue::

        sage: it = iter_pisot_irreductible(3, (5,10))
        sage: for _ in range(4): next(it)
        WordMorphism: 0->0000001, 1->2, 2->0
        WordMorphism: 0->0000002, 1->0, 2->1
        WordMorphism: 0->0000010, 1->2, 2->0
        WordMorphism: 0->0000012, 1->0, 2->1
    """
    from slabbe.matrices import is_pisot
    W = FiniteWords(range(d))
    for m in W.iter_morphisms(arg):
        incidence_matrix = m.incidence_matrix()
        if not incidence_matrix.det() == 1:
            continue
        if not m.is_primitive(): # mathematiquement non necessaire
            continue
        if not is_pisot(incidence_matrix):
            continue
        yield m

def is_left_marked(m):
    r"""
    EXAMPLES::

        sage: from slabbe.word_morphisms import is_left_marked
        sage: m = WordMorphism('0->00001,1->00010')
        sage: is_left_marked(m)
        True
        sage: m = WordMorphism('0->00001,1->00001')
        sage: is_left_marked(m)
        False
        sage: m = WordMorphism('0->00001,1->00001,2->201')
        sage: is_left_marked(m)
        False
        sage: m = WordMorphism('0->00001,1->10001,2->201')
        sage: is_left_marked(m)
        True
        sage: m = WordMorphism('0->000001,1->010001,2->0201')
        sage: is_left_marked(m)
        True
        sage: m = WordMorphism('0->000001,1->010001,2->0101')
        sage: is_left_marked(m)
        False
    """
    images = m.images()
    N = len(images)
    L = [len(image) for image in images]
    for i in range(2*max(L)):
        s = len(set(image[i % L[a]] for (a,image) in enumerate(images)))
        if s == N:
            return True
        elif s > 1:
            return False
    # is cyclic
    return False

def is_marked(m):
    r"""
    EXAMPLES::

        sage: from slabbe.word_morphisms import is_marked
        sage: m = WordMorphism('0->00001,1->00010')
        sage: is_marked(m)
        True
    """
    return is_left_marked(m) and is_left_marked(m.reversal())

def iter_primitive_marked_classP_morphisms(words, n):
    r"""
    EXAMPLES::

        sage: from slabbe.word_morphisms import iter_primitive_marked_classP_morphisms
        sage: F = FiniteWords('ab')
        sage: it = iter_primitive_marked_classP_morphisms(F, 2)
        sage: list(it)
        []
        sage: it = iter_primitive_marked_classP_morphisms(F, 3)
        sage: list(it)
        [WordMorphism: a->ab, b->a, WordMorphism: a->b, b->ba]
        sage: it = iter_primitive_marked_classP_morphisms(F, 4)
        sage: list(it)
        [WordMorphism: a->aba, b->a,
         WordMorphism: a->bab, b->a,
         WordMorphism: a->b, b->aba,
         WordMorphism: a->b, b->bab,
         WordMorphism: a->abb, b->a,
         WordMorphism: a->ab, b->aa,
         WordMorphism: a->bb, b->ba,
         WordMorphism: a->b, b->baa]
    """
    for m in iter_conjugate_classP(words, n):
        if m.is_primitive() and is_marked(m):
            yield m

def desubstitute_prefix_code(self, u):
    r"""
    Return the preimage of u under self.

    INPUT:

    - ``self`` -- word morphism, a prefix code
    - ``u`` -- word

    EXAMPLES::

        sage: from slabbe.word_morphisms import desubstitute_prefix_code
        sage: s = WordMorphism({0:[0,1],1:[1,0]})
        sage: w = desubstitute_prefix_code(s, Word([0,1,0,1,1,0]))
        sage: w
        word: 001

    The result lives in the domain of the given substitution::

        sage: w.parent()
        Finite words over {0, 1}

    TESTS::

        sage: s = WordMorphism({0:[0,1],1:[1,0],2:[1,0]})
        sage: desubstitute_prefix_code(s, Word([0,1,0,1,1,0]))
        Traceback (most recent call last):
        ...
        ValueError: non unique desubstitution, m(1)=10, m(2)=10 are prefixes of u[4:]

    ::

        sage: s = WordMorphism({0:[0,1],1:[1,0]})
        sage: desubstitute_prefix_code(s, Word([0,1,0,1,1,1]))
        Traceback (most recent call last):
        ...
        ValueError: desubstitution is impossible for u[4:]
    """
    morph = self._morph
    i = 0
    len_u = len(u)
    result = []
    while i < len_u:
        ui = u[i:]
        keys = [(k,v) for k,v in morph.items() if v.is_prefix(ui)]
        if len(keys) > 1:
            s = ", ".join(["m({})={}".format(k,v) for (k,v) in keys])
            msg = ("non unique desubstitution, "
                   "{} are prefixes of u[{}:] ".format(s,i))
            raise ValueError(msg)
        if len(keys) == 0:
            raise ValueError("desubstitution is impossible for u[{}:]".format(i))
        k,v = keys[0]
        result.append(k)
        i += len(v)
    W = self.domain()
    return W(result)

def desubstitute(self, u):
    r"""
    EXAMPLES:

    Unique preimage::

        sage: from slabbe.word_morphisms import desubstitute
        sage: s = WordMorphism({0:[0,1],1:[1,0]})
        sage: desubstitute(s, Word([0,1,0,1,1,0]))
        [word: 001]

    Non-unique preimage::

        sage: s = WordMorphism({0:[0,1],1:[1,0],2:[1,0]})
        sage: desubstitute(s, Word([0,1,0,1,1,0]))
        [word: 001, word: 002]

    No preimage::

        sage: s = WordMorphism({0:[0,1],1:[1,0]})
        sage: desubstitute(s, Word([0,1,0,1,1,1]))
        []

    Lot of preimages (computation is done in parallel with Florent's Hivert
    parallel map reduce code)::

        sage: s = WordMorphism({0:[0,1],1:[0,1]})
        sage: w = Word([0,1]) ^ 10
        sage: L = desubstitute(s, w)
        sage: len(L)
        1024
    """
    morph = self._morph
    len_u = len(u)
    def successor(node):
        L,i = node
        if i == len_u:
            return []
        else:
            ui = u[i:]
            return [(L+[k],i+len(v)) for k,v in morph.items() if v.is_prefix(ui)]
    roots = [([],0)]
    post_process = lambda node:node if node[1]==len_u else None
    from sage.sets.recursively_enumerated_set import RecursivelyEnumeratedSet
    R = RecursivelyEnumeratedSet(roots, successor, structure='forest',
            post_process=post_process)
    W = self.domain()
    map_function = lambda node:[W(node[0])]
    reduce_function = lambda x,y:x+y
    reduce_init = []
    return R.map_reduce(map_function, reduce_function, reduce_init)

def return_substitution(self, u, coding=False, length=1000):
    r"""
    Return the return substitution of self according to factor u.

    INPUT:

    - ``self`` -- word morphism
    - ``u`` -- word such that u is a prefix of self(u)
    - ``coding`` -- boolean (default: ``False``), whether to
      include the return word coding morphism
    - ``length`` -- integer (default: ``1000``), compute the first 1000 letters
      of the derived sequence to make sure every return word are seen

    EXAMPLES::

        sage: from slabbe.word_morphisms import return_substitution
        sage: s = WordMorphism({0:[0,1],1:[1,0]})
        sage: return_substitution(s, Word([0]))
        WordMorphism: 0->012, 1->02, 2->1
        sage: return_substitution(s, Word([0,1]))
        WordMorphism: 0->01, 1->23, 2->013, 3->2
        sage: return_substitution(s, Word([0,1,1]))
        WordMorphism: 0->01, 1->23, 2->013, 3->2

    ::

        sage: return_substitution(s, Word([0]), True)
        (WordMorphism: 0->012, 1->02, 2->1, 
         WordMorphism: 0->011, 1->01, 2->0)
        sage: return_substitution(s, Word([0,1]), True)
        (WordMorphism: 0->01, 1->23, 2->013, 3->2,
         WordMorphism: 0->011, 1->010, 2->0110, 3->01)

    ::

        sage: s = WordMorphism({0:[0,0,1],1:[0,1]})
        sage: return_substitution(s, Word([0]))
        WordMorphism: 0->01, 1->011

    TESTS::

        sage: s = WordMorphism({0:[0,1],1:[1,0]})
        sage: sigma_u, theta_u = return_substitution(s, Word([0]), coding=True)
        sage: sigma_u
        WordMorphism: 0->012, 1->02, 2->1
        sage: theta_u
        WordMorphism: 0->011, 1->01, 2->0
        sage: theta_u*sigma_u == s*theta_u
        True
        sage: theta_u*sigma_u
        WordMorphism: 0->011010, 1->0110, 2->01
    """
    from slabbe.infinite_word import derived_sequence
    a = u[0]
    x = self.fixed_point(a)
    s, D = derived_sequence(x, u, coding=True)
    _ = s[length] # make sure that D is complete (exact value 
                  # is known by J. Leroy and F. Durand)
    code_to_return_word = WordMorphism({v:k for k,v in D.items()})
    rep = {}
    for key,value in D.items():
        self_key = self(key)
        L = desubstitute(code_to_return_word, self_key)
        if len(L) == 0:
            raise ValueError("desubstitution of {} by {} "
                     "is impossible ".format(self_key, code_to_return_word))
        elif len(L) > 1:
            s = "=".join(["m({})".format(u) for u in L])
            msg = ("non unique desubstitution, "
                   "{}={}".format(s,self_key))
            raise ValueError(msg)
        #print(key,value,self_key,L[0])
        preimage = L[0]
        rep[value] = preimage
    m = WordMorphism(rep) 
    if coding:
        return m, code_to_return_word
    else:
        return m

def compute_xsi(self, u):
    r"""
    EXAMPLES::

        sage: from slabbe.word_morphisms import compute_xsi
        sage: s = WordMorphism({0:[0,1],1:[1,0]})
        sage: compute_xsi(s, Word([0]))
        sigma_u= 0->012, 1->02, 2->1
        theta_u= 0->011, 1->01, 2->0
        psi= 0->(0, 0),(0, 1),(0, 2), 1->(1, 0),(1, 1), 2->(2, 0)
        psi*sigma_u= 0->(0, 0),(0, 1),(0, 2),(1, 0),(1, 1),(2, 0), 1->(0, 0),(0, 1),(0, 2),(2, 0), 2->(1, 0),(1, 1)
        Finite words over {(0, 0), (0, 1), (0, 2), (1, 0), (1, 1), (2, 0)}
        [1 0 0]
        [1 0 0]
        [1 0 0]
        [0 1 0]
        [0 1 0]
        [0 0 1]
        We want zeta such that:
        zeta((0, 0),(0, 1),(0, 2)) = (0, 0),(0, 1),(0, 2),(1, 0),(1, 1),(2, 0)
        zeta((1, 0),(1, 1)) = (0, 0),(0, 1),(0, 2),(2, 0)
        zeta((2, 0)) = (1, 0),(1, 1)
    """
    sigma_u, theta_u = return_substitution(self, u, coding=True)
    assert theta_u*sigma_u == self*theta_u, "identity is not verified"
    print("sigma_u=", sigma_u)
    print("theta_u=", theta_u)
    d = {k:[(k,i) for i in range(len(v))] for k,v in theta_u._morph.items()}
    psi = WordMorphism(d)
    print("psi=", psi)
    print("psi*sigma_u=", psi*sigma_u)
    print(psi.codomain())
    print(psi.incidence_matrix())
    print("We want zeta such that:")
    for k,v in psi._morph.items():
        print("zeta({}) = {}".format(v, psi(sigma_u(k))))





# -*- coding: utf-8 -*-
r"""
A time evolution picture of packages built by Sage

EXAMPLES::

    sage: from slabbe.analyze_sage_build import draw_sage_build
    sage: t = draw_sage_build()
    sage: _ = t.pdf(view=False)

AUTHOR:

    - Sébastien Labbé, December 9-11, 2016
"""
#*****************************************************************************
#       Copyright (C) 2016 Sébastien Labbé <slabqc@gmail.com>
#
#  Distributed under the terms of the GNU General Public License version 2 (GPLv2)
#
#  The full text of the GPLv2 is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************
from __future__ import absolute_import, print_function
import os, datetime, re
from slabbe.tikz_picture import TikzPicture
from pytimeparse.timeparse import timeparse

def last_modified_datetime(path_to_file):
    r"""
    EXAMPLES::

        sage: from slabbe.analyze_sage_build import last_modified_datetime
        sage: from sage.env import SAGE_ROOT
        sage: last_modified_datetime(SAGE_ROOT+'/README.md')
        datetime.datetime(..., ..., ..., ..., ..., ...)
        sage: last_modified_datetime(SAGE_ROOT+'/VERSION.txt')
        datetime.datetime(..., ..., ..., ..., ..., ...)
    """
    t_stamp = os.path.getmtime(path_to_file)
    return datetime.datetime.fromtimestamp(t_stamp)

def build_duration_logs(path_to_file, pattern=None):
    r"""
    INPUT:

    - ``path_to_file`` -- string, file to parse
    - ``pattern`` -- string or ``None``, string to parse. If ``None``, it
      returns 0 seconds.

    OUTPUT:

        list of timedelta objects

    EXAMPLES::

        sage: L = [ ('/../ptestlong.log', 'Total time for all tests: '),
        ....:       ('/../dochtml.log', 'Elapsed time: '),
        ....:       ('/../start.log', None),
        ....:       ('/sagelib-7.5.beta6.log', 'real\t'),
        ....:       ('/sqlite.log', 'real\t')]

    ::

        sage: from slabbe.analyze_sage_build import build_duration_logs
        sage: from sage.env import SAGE_ROOT
        sage: import os
        sage: SAGE_LOGS_PKGS = os.path.join(SAGE_ROOT,'logs','pkgs')
        sage: import os
        sage: for (file, pattern) in L:                  # random
        ....:     print(file)
        ....:     if not os.path.exists(SAGE_LOGS_PKGS+file):
        ....:         continue
        ....:     build_duration_logs(SAGE_LOGS_PKGS+file, pattern)
        /../ptestlong.log
        /../dochtml.log
        [datetime.timedelta(0, 471, 700000)]
        /../start.log
        []
        /sagelib-7.5.beta6.log
        /sqlite.log
        []

    """
    if pattern is None:
        return []

    with open(path_to_file, 'r') as f:
        s = f.read()

    result = re.findall('^'+pattern+'.*', s, flags=re.MULTILINE)

    # line = "real\t0m41.027s"            # in logs/*.log
    # line = "Total time for all tests: 14922.7 seconds" # in ptestlong.log
    # line = "Elapsed time: 4156.8 seconds." # in dochtml.log
    L = []
    for line in result:
        time_string = line[len(pattern):].strip('.')
        seconds = timeparse(time_string)
        if seconds is None:
            raise ValueError('Unable to parse time delta in string '
                    '"{}" of file {}'.format(time_string, path_to_file))
        L.append(datetime.timedelta(seconds=seconds))

    return L

def sage_logs_datetime_list(consider='last', verbose=False):
    r"""
    Return a dictionnary of duration and last modified information from the
    sage log files.

    INPUT:

    - ``consider`` - string (default: ``'last'``), ``'last'`` or ``'all'``, in
      any log file, consider the last duration found or the sum of all of them
    - ``verbose`` - boolean (default: ``False``)

    EXAMPLES::

        sage: import datetime
        sage: from slabbe.analyze_sage_build import sage_logs_datetime_list
        sage: L = sage_logs_datetime_list()
        sage: L = sage_logs_datetime_list()
        sage: stop = datetime.datetime.now()
        sage: start = stop - datetime.timedelta(14r)
        sage: L_filtered = [(A,B,delta,file) for (A,B,delta,file) in L
        ....:                  if start <= A and B <= stop]
    """
    from sage.env import SAGE_ROOT
    SAGE_LOGS = os.path.join(SAGE_ROOT,'logs')
    SAGE_LOGS_PKGS = os.path.join(SAGE_ROOT,'logs','pkgs')
    K = []
    pattern = 'real\t'
    for file in os.listdir(SAGE_LOGS_PKGS):
        path_to_file = os.path.join(SAGE_LOGS_PKGS, file)
        K.append((path_to_file, pattern))

    path_to_file = os.path.join(SAGE_LOGS, 'dochtml.log')
    if os.path.exists(path_to_file):
        pattern = 'Elapsed time: '
        K.append((path_to_file, pattern))

    path_to_file = os.path.join(SAGE_LOGS, 'ptestlong.log')
    if os.path.exists(path_to_file):
        pattern = 'Total time for all tests: '
        K.append((path_to_file, pattern))

    path_to_file = os.path.join(SAGE_LOGS, 'start.log')
    if os.path.exists(path_to_file):
        K.append((path_to_file, None))

    L = []
    for path_to_file, pattern in K:
        file = os.path.split(path_to_file)[-1]
        B = last_modified_datetime(path_to_file)
        deltas = build_duration_logs(path_to_file, pattern)
        if pattern is None:
            delta = datetime.timedelta(0)
        elif not deltas:
            if verbose:
                print('Warning: no duration found in file {} '
                      'with pattern "{}"'.format(file, pattern))
            delta = datetime.timedelta(0)
        elif consider == 'last':
            delta = deltas[-1]
        elif consider == 'all':
            delta = sum(deltas, datetime.timedelta(0))
        else:
            raise ValueError('Unknown value for consider(={})'.format(consider))
        entry = (B-delta,B,delta,file)
        L.append(entry)

    return L

def draw_sage_build(start=None, stop=None, consider='last', verbose=False):
    r"""
    Return a time evolution picture of packages built by Sage

    INPUT:

    - ``start`` -- datetime object (default is January 1st, 2000)
    - ``stop`` -- datetime object (default is now)
    - ``consider`` - string (default: ``'last'``), ``'last'`` or ``'all'``, in
      any log file, consider the last duration found or the sum of all of them
    - ``verbose`` - boolean (default: ``False``)

    OUTPUT:

    TikzPicture object

    EXAMPLES::

        sage: from slabbe.analyze_sage_build import draw_sage_build
        sage: t = draw_sage_build()

    During the previous 7 day::

        sage: import datetime
        sage: stop = datetime.datetime.now()
        sage: start = stop - datetime.timedelta(7r)
        sage: t = draw_sage_build(start, stop)        # not tested

    ::

        sage: t = draw_sage_build(stop=datetime.datetime(2016,12,9)) # not tested

    TESTS:

    It may fail if nothing is found during the period::

        sage: t = draw_sage_build()  # not tested
        Traceback (most recent call last):
        ...
        ValueError: no package found built between start date (=2016-12-08
        16:58:35.189127) and stop date (=2016-12-09 16:58:35.189127). Oldest is
        2016-02-16 11:38:00.673327. Newest is 2016-12-08 11:27:03.312340.
    """
    if stop is None:
        stop = datetime.datetime.now()
    if start is None:
        #start = stop - datetime.timedelta(1) # minus 1 day
        start = datetime.datetime(2000, 1, 1)
    assert start < stop, ("start date (={}) must precede "
                          "stop date (={})".format(start, stop))

    # Obtain datetime and timedelta from logs
    L_all = sage_logs_datetime_list(consider=consider, verbose=verbose)
    if not L_all:
        raise ValueError("no package found in the logs directory")
    L_all.sort()

    # Filter data according to desired period
    L = [(A,B,delta,file) for (A,B,delta,file) in L_all
                          if start <= A and B <= stop]
    if not L:
        raise ValueError("no package found built between start date (={}) "
                         "and stop date (={}). Oldest started at {}. "
                         "Newest started at {}.".format(start, stop,
                                        L_all[0][0], L_all[-1][0]))
    # time to x axis float
    effective_start = min(A for (A,B,delta,file) in L)
    effective_stop = max(B for (A,B,delta,file) in L)
    duration = (effective_stop-effective_start)
    TOTAL = duration.total_seconds()
    if TOTAL == 0:
        TOTAL += 1 # avoid division by zero

    def timedelta_to_float(t):
        return t.total_seconds() / TOTAL * 40
    def datetime_to_float(t):
        return timedelta_to_float(t-effective_start)

    # Tikz code
    lines = []
    lines.append('\\begin{tikzpicture}[scale=0.5]')
    str_rect = '\\draw[fill=orange] ({}, {}) rectangle ({}, {}) ;'
    str_node = '\\node[{}] at ({}, {}) {{{}}};'

    # Titles
    lines.append(str_node.format('right',0,5,
                                 effective_start.isoformat()+' (start)'))
    lines.append(str_node.format('right',0,4,
                                 effective_stop.isoformat()+' (end)'))
    lines.append(str_node.format('right',0,3,
                                 '{} (duration)'.format(duration)))

    # Axis
    t0 = datetime_to_float(effective_start)
    tn = datetime_to_float(effective_stop)
    axis_y = 1.5
    lines.append('\\draw[thick,->] ({},{axis_y}) -- '
                                  '({},{axis_y});'.format(t0, tn, axis_y=axis_y))
    duration_minutes = int(duration.total_seconds()/60)
    step = duration_minutes // 4
    step = 10 * (step // 10 + 1)
    for n_minutes in range(0, duration_minutes, step):
        delta = datetime.timedelta(0,n_minutes*60)
        tj = timedelta_to_float(delta)
        lines.append('\\draw[thick] ({tj},{}) -- ({tj},{}) node[below] '
                '{{{delta}}};'.format(axis_y+.3, axis_y-.3, delta=delta, tj=tj))

    # Boxes
    L.sort()
    for i,entry in enumerate(L):
        A,B,delta,file_ext = entry
        Af = datetime_to_float(A)
        Bf = datetime_to_float(B)
        lines.append(str_rect.format(Af,-i,Bf,-i-1))
        file,ext = os.path.splitext(file_ext)
        file = file.replace('_','\\_')
        file = '\\texttt{{{}}}'.format(file)
        lines.append(str_node.format('right',Bf,-i-.5,file))

    lines.append('\\end{tikzpicture}')
    return TikzPicture('\n'.join(lines))




# -*- coding: utf-8 -*-
r"""
Combinatorics methods

EXEMPLES::

    sage: from slabbe.combinat import random_composition
    sage: c = random_composition(24,9)
    sage: c        # random
    [1, 4, 2, 2, 3, 5, 4, 2, 1]

::

    sage: from slabbe.combinat import random_simplex_point
    sage: random_simplex_point(3)      # random
    [0.2493321790694003, 0.5353600549544871, 0.21530776597611256]

::

    sage: from slabbe.combinat import random_interior_point
    sage: p = polytopes.hypercube(3)
    sage: p = p + vector([20,0,0])
    sage: random_interior_point(p)          # random
    (19.33174562788114, -0.5428002756082744, -0.3568284089832092)
"""
#*****************************************************************************
#       Copyright (C) 2016 Sébastien Labbé <slabqc@gmail.com>
#
#  Distributed under the terms of the GNU General Public License version 2 (GPLv2)
#
#  The full text of the GPLv2 is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************
from __future__ import absolute_import, print_function
from random import randint, random, randrange

def non_uniform_randint(L):
    r"""
    Return a random integer from 0 to len(L)-1 with probabilities
    proportional to the (integral) entries of L.

    INPUT:

    - ``L`` -- list of integers

    EXEMPLES::

        sage: from slabbe.combinat import non_uniform_randint
        sage: non_uniform_randint([2,3,5])    # random
        1
        sage: non_uniform_randint([2,3,5])    # random
        2
        sage: non_uniform_randint([2,3,5])    # random
        2
        sage: from collections import Counter
        sage: L = [non_uniform_randint([2,3,5]) for _ in range(100000)]
        sage: Counter(L)            # random
        Counter({2: 49805, 1: 30228, 0: 19967})
    """
    sum_L = sum(L)
    R = randrange(sum_L)
    for i,a in enumerate(L):
        R -= a
        if R < 0:
            return i
    else:
        assert False, "problem, algorithm should not reach this point"

def random_composition(n, length):
    r"""
    EXEMPLES::

        sage: from slabbe.combinat import random_composition
        sage: random_composition(4,2)   # random
        [1, 3]
        sage: random_composition(4,2)   # random
        [2, 2]
        sage: random_composition(4,2)   # random
        [3, 1]
        sage: c = random_composition(24,9)
        sage: c  # random
        [1, 4, 2, 2, 3, 5, 4, 2, 1]
        sage: sum(c)
        24

    Because this is very slow!!::

        sage: C = Compositions(24, length=9)
        sage: %time C.random_element()     # not tested
        CPU times: user 43.3 s, sys: 31.9 ms, total: 43.3 s
        Wall time: 43.2 s
        [2, 2, 5, 2, 8, 1, 1, 2, 1]
    """
    L = [randint(0,n-length) for _ in range(length-1)]
    L.sort()
    L = [a+i+1 for i,a in enumerate(L)]
    L.insert(0,0)
    L.append(n)
    return [L[i+1]-L[i] for i in range(length)]

def random_simplex_point(d):
    r"""
    Return a random vector of d positive real numbers summing to 1.

    INPUT:

    - ``d`` -- integer

    EXEMPLES::

        sage: from slabbe.combinat import random_simplex_point
        sage: random_simplex_point(7)          # random
        [0.06137280030263492,
         0.08066113584919432,
         0.09019666554921013,
         0.24473802319989957,
         0.41761622259683495,
         0.10043545384643937,
         0.004979698655786735]
        sage: random_simplex_point(2)          # random
        [0.5677654878488222, 0.4322345121511778]
        sage: random_simplex_point(3)          # random
        [0.2493321790694003, 0.5353600549544871, 0.21530776597611256]

    TESTS::

        sage: sum(random_simplex_point(4))
        1.0
        sage: sum(random_simplex_point(7))
        1.0
    """
    L = [random() for _ in range(d-1)]
    L.append(0)
    L.append(1)
    L.sort()
    return [L[i+1]-L[i] for i in range(d)]

def random_interior_point(self, a=10, integer=False):
    r"""
    Return a random interior point of a polytope.

    INPUT:

    - ``a`` -- number, amplitude of random deplacement in the direction of each
      ray.
    - ``integer`` -- bool, whether the output must be with integer
      coordinates

    EXEMPLES::

        sage: from slabbe.combinat import random_interior_point
        sage: p = polytopes.hypercube(3)
        sage: p = p + vector([20,0,0])
        sage: p.center()
        (20, 0, 0)
        sage: random_interior_point(p)     # random
        (19.33174562788114, -0.5428002756082744, -0.3568284089832092)
        sage: random_interior_point(p)     # random
        (20.039169786976075, -0.4121594862234468, -0.05623023234688396)
        sage: random_interior_point(p, integer=True)     # random
        (21, 0, 0)
    """
    from sage.geometry.polyhedron.constructor import Polyhedron
    L = list(self.vertices())
    L.extend(a*random()*ray.vector() for ray in self.rays())
    P = Polyhedron(L)
    return random_interior_point_compact_polytope(P)

def random_interior_point_compact_polytope(self, uniform='simplex', integer=False):
    r"""
    Return a random interior point of a compact polytope.

    INPUT:

    - ``uniform`` -- ``'points'`` (slow) or ``'simplex'`` (fast), whether
      to take the probability uniformly with respect to the set of integral
      points or with respect to the simplexes.
    - ``integer`` -- bool, whether the output must be with integer
      coordinates

    EXEMPLES::

        sage: from slabbe.combinat import random_interior_point_compact_polytope
        sage: p = polytopes.hypercube(3)
        sage: p = p + vector([30,20,10])
        sage: p.center()
        (30, 20, 10)
        sage: random_interior_point_compact_polytope(p)     # random
        (19.33174562788114, -0.5428002756082744, -0.3568284089832092)
        sage: random_interior_point_compact_polytope(p)     # random
        (20.039169786976075, -0.4121594862234468, -0.05623023234688396)
        sage: random_interior_point_compact_polytope(p, integer=True) # random
        (30, 19, 9)
    """
    assert self.is_compact(), "input is not compact"
    from sage.geometry.polyhedron.constructor import Polyhedron
    triangulation = self.triangulate()
    T = []
    for simplex_indices in self.triangulate():
        simplex_vertices = [self.Vrepresentation(i) for i in simplex_indices]
        simplex = Polyhedron(simplex_vertices)
        T.append(simplex)
    if uniform == 'points':
        v = [p.integral_points_count() for p in T] # slow
        i = non_uniform_randint(v)
    elif uniform == 'simplex':
        i = randrange(len(T))
    else:
        raise ValueError('unknown value for uniform(={})'.format(uniform))
    return random_interior_point_simplex(T[i], integer=integer)
def random_interior_point_simplex(self, integer=False):
    r"""
    Return a random interior point of a simplex.

    This method was based on the code ``P.center()`` of sage.

    INPUT:

    - ``integer`` -- bool, whether the output must be with integer
      coordinates

    EXEMPLES::

        sage: from slabbe.combinat import random_interior_point_simplex
        sage: P = 10 * polytopes.simplex(3)
        sage: random_interior_point_simplex(P)          # random
        (2.8787864522849462, 5.302173919578364, 1.7059355910006113, 0.11310403713607808)
        sage: a = random_interior_point_simplex(P, integer=True)
        sage: a             # random
        (0, 7, 1, 2)
        sage: a in P
        True
    """
    from sage.modules.free_module_element import vector
    from sage.misc.functional import round
    from sage.functions.generalized import sign
    assert self.is_simplex(), "input is not a simplex"
    d = self.n_vertices()
    R = random_simplex_point(d)
    vertex_sum = vector(self.base_ring(), [0]*self.ambient_dim())
    for v,r in zip(self.vertex_generator(), R):
        vertex_sum += r*v.vector()
    if integer:
        v = vector(map(round, vertex_sum))
        r = round(sum(vertex_sum) - sum(v))
        s = sign(r)
        for _ in range(abs(r)):
            v[randrange(len(v))] += s
        v.set_immutable()
        return v
    else:
        vertex_sum.set_immutable()
        return vertex_sum

def integral_points_count_union_of_polytopes(L):
    r"""
    Return the cardinality of an union of polytopes.

    See https://en.wikipedia.org/wiki/Inclusion–exclusion_principle

    INPUT:

    - ``L`` -- list of polytopes

    EXEMPLES::

        sage: P = Polyhedron(ieqs=[[0,1,0],[0,0,1],[9,-1,0],[9,0,-1]])
        sage: Q = Polyhedron(ieqs=[[-5,1,0],[-5,0,1],[14,-1,0],[14,0,-1]])
        sage: P.integral_points_count()   # optional -- latte_int
        100
        sage: Q.integral_points_count()   # optional -- latte_int
        100
        sage: from slabbe.combinat import integral_points_count_union_of_polytopes
        sage: integral_points_count_union_of_polytopes([P,Q])  # optional -- latte_int
        175
    """
    from sage.combinat.subset import Subsets
    n = len(L)
    range_n = range(n)
    res = 0
    for k in range_n:
        sgn = (-1)**k
        for indices in Subsets(range_n, k+1):
            I = intersection_of_polytopes(L[i] for i in indices)
            try:
                card = I.integral_points_count()
            except IndexError:
                print("IndexError when indices={}, see https://trac.sagemath.org/ticket/21491".format(indices))
                card = 0
            res += sgn * card
    return res

def intersection_of_polytopes(L):
    r"""
    Return the intersection of a list of polytopes.

    INPUT:

    - ``L`` -- list of polytopes

    EXEMPLES::

        sage: from slabbe.combinat import intersection_of_polytopes
        sage: P = Polyhedron(ieqs=[[0,1,0],[0,0,1],[9,-1,0],[9,0,-1]])
        sage: Q = Polyhedron(ieqs=[[-5,1,0],[-5,0,1],[14,-1,0],[14,0,-1]])
        sage: I = intersection_of_polytopes([P,Q])
        sage: I
        A 2-dimensional polyhedron in QQ^2 defined as the convex hull of 4 vertices
        sage: I.integral_points_count()      # optional -- latte_int
        25

    TESTS::

        sage: intersection_of_polytopes(iter([P,Q]))
        A 2-dimensional polyhedron in QQ^2 defined as the convex hull of 4 vertices
        sage: intersection_of_polytopes(iter([]))
        Traceback (most recent call last):
        ...
        NotImplementedError: intersection of an empty list of polytopes not defined
    """
    it = iter(L)
    try:
        rep = next(it)
    except StopIteration:
        raise NotImplementedError("intersection of an empty list of polytopes not defined")
    for p in it:
        rep = rep.intersection(p)
    return rep

